(function ($, undefined) {

    var ms = $.mobiscroll,
        util = ms.util,
        isString = util.isString,
        defaults = {
            inputClass: '',
            invalid: [],
            rtl: false,
            showInput: true,
            group: false,
            groupLabel: 'Groups',
            checkIcon: 'checkmark',
            dataText: 'text',
            dataValue: 'value',
            dataGroup: 'group',
            dataDisabled: 'disabled'
        };

    ms.presetShort('select');

    ms.presets.scroller.select = function (inst) {
        var change,
            group,
            groupWheelIdx,
            headerText,
            i,
            input,
            optionArray,
            optionWheelIdx,
            option,
            origValues,
            prevGroup,
            timer,
            orig = $.extend({}, inst.settings),
            s = $.extend(inst.settings, defaults, orig),
            layout = s.layout || (/top|bottom/.test(s.display) ? 'liquid' : ''),
            isLiquid = layout == 'liquid',
            elm = $(this),
            multiple = s.multiple || elm.prop('multiple'),
            id = this.id + '_dummy',
            lbl = $('label[for="' + this.id + '"]').attr('for', id),
            label = s.label !== undefined ? s.label : (lbl.length ? lbl.text() : elm.attr('name')),
            selectedClass = 'dw-msel mbsc-ic mbsc-ic-' + s.checkIcon,
            origReadOnly = s.readonly,
            data = s.data,
            hasData = !!data,
            hasGroups = hasData ? data[0][s.dataGroup] : $('optgroup', elm).length,
            defaultValue = hasData ? data[0][s.dataValue] : $('option', elm).attr('value'),
            groupWheel = hasGroups && s.group,
            groupHdr = hasGroups && !groupWheel,
            values = elm.val() || [],
            invalid = [],
            selectedValues = {},
            options = {},
            groups = {};

        function prepareData() {
            var gr,
                lbl,
                opt,
                txt,
                val,
                c = 0,
                groupIndexes = {};

            if (hasData) {
                $.each(data, function (i, v) {
                    txt = v[s.dataText];
                    val = v[s.dataValue];
                    lbl = v[s.dataGroup];
                    opt = {
                        value: val,
                        text: txt
                    };
                    options[val] = opt;

                    if (hasGroups) {
                        if (groupIndexes[lbl] === undefined) {
                            gr = { label: lbl, options: [] };
                            groups[c] = gr;
                            groupIndexes[lbl] = c;
                            c++;
                        } else {
                            gr = groups[groupIndexes[lbl]];
                        }
                        opt.group = groupIndexes[lbl];
                        gr.options.push(opt);
                    }
                    if (v[s.dataDisabled]) {
                        invalid.push(val);
                    }
                });
            } else {
                if (hasGroups) {
                    $('optgroup', elm).each(function (i) {
                        groups[i] = { label: this.label, options: [] };
                        $('option', this).each(function () {
                            opt = {
                                value: this.value,
                                text: this.text,
                                group: i
                            };
                            options[this.value] = opt;
                            groups[i].options.push(opt);
                            if (this.disabled) {
                                invalid.push(this.value);
                            }
                        });
                    });
                } else {
                    optionArray = [];
                    $('option', elm).each(function () {
                        opt = {
                            value: this.value,
                            text: this.text
                        };
                        options[this.value] = opt;
                        optionArray.push(opt);
                        if (this.disabled) {
                            invalid.push(this.value);
                        }
                    });
                }
            }
        }

        function genValues(data, keys, values) {
            $.each(data, function (i, v) {
                values.push(v.text);
                keys.push(v.value);
            });
        }

        function genWheels() {
            var wheel,
                wg = 0,
                values = [],
                keys = [],
                w = [[]];

            // Generate group wheel
            if (groupWheel) {
                $.each(groups, function (i, v) {
                    values.push(v.label);
                    keys.push(i);
                });

                wheel = {
                    values: values,
                    keys: keys,
                    label: s.groupLabel
                };

                if (isLiquid) {
                    w[0][wg] = wheel;
                } else {
                    w[wg] = [wheel];
                }

                wg++;
            }

            values = [];
            keys = [];

            if (groupHdr) { // Generate options wheel with group headers
                $.each(groups, function (i, gr) {
                    values.push(gr.label);
                    keys.push('__group' + i);
                    invalid.push('__group' + i);
                    genValues(gr.options, keys, values);
                });
            } else { // Generate options wheel (for selected group or all, if not grouped)
                genValues(groupWheel ? groups[group].options : optionArray, keys, values);
            }

            wheel = {
                multiple: multiple,
                values: values,
                keys: keys,
                label: label
            };

            if (isLiquid) {
                w[0][wg] = wheel;
            } else {
                w[wg] = [wheel];
            }

            return w;
        }

        function getOption(v) {
            option = multiple ? (v ? v[0] : defaultValue) : (v === undefined || v === null || v === '' ? defaultValue : v);

            if (groupWheel) {
                group = options[option].group;
            }
        }

        function getVal(temp, group) {
            var val = temp ? inst._tempWheelArray : (inst._hasValue ? inst._wheelArray : null);
            return val ? (s.group && group ? val : val[optionWheelIdx]) : null;
        }

        function onFill(v, change) {
            var txt,
                val,
                sel = [],
                i = 0;

            if (multiple) {
                val = [];

                for (i in selectedValues) {
                    sel.push(options[i].text);
                    val.push(i);
                }

                txt = sel.join(', ');
            } else {
                val = v;
                txt = options[v] ? options[v].text : '';
            }

            inst._tempValue = val;

            input.val(txt);
            elm.val(val);

            if (change) {
                inst._preventChange = true;
                elm.change();
            }
        }

        function onTap(li) {
            var val = li.attr('data-val'),
                selected = li.hasClass('dw-msel');

            if (multiple && li.closest('.dwwl').hasClass('dwwms')) {
                if (li.hasClass('dw-v')) {
                    if (selected) {
                        li.removeClass(selectedClass).removeAttr('aria-selected');
                        delete selectedValues[val];
                    } else {
                        li.addClass(selectedClass).attr('aria-selected', 'true');
                        selectedValues[val] = val;
                    }
                }
                return false;
            }
        }

        if (!s.invalid.length) {
            s.invalid = invalid;
        }

        if (groupWheel) {
            groupWheelIdx = 0;
            optionWheelIdx = 1;
        } else {
            groupWheelIdx = -1;
            optionWheelIdx = 0;
        }

        if (multiple) {
            if (values && isString(values)) {
                values = values.split(',');
            }
            for (i = 0; i < values.length; i++) {
                selectedValues[values[i]] = values[i];
            }
        }

        prepareData();

        getOption(elm.val());

        $('#' + id).remove();

        input = $('<input type="text" id="' + id + '" class="' + s.inputClass + '" placeholder="' + (s.placeholder || '') + '" readonly />');

        if (s.showInput) {
            input.insertBefore(elm);
        }

        inst.attachShow(input);

        elm.addClass('dw-hsel').attr('tabindex', -1).closest('.ui-field-contain').trigger('create');

        onFill(option);

        // Extended methods
        // ---

        inst.setVal = function (val, fill, change, temp, time) {
            if (multiple) {
                if (val && isString(val)) {
                    val = val.split(',');
                }
                selectedValues = util.arrayToObject(val);
                val = val ? val[0] : null;
            }
            inst._setVal(val, fill, change, temp, time);
        };

        inst.getVal = function (temp, group) {
            if (multiple) {
                return util.objectToArray(selectedValues);
            }
            return getVal(temp, group);
        };

        // @deprecated since 2.14.0, backward compatibility code
        // ---
        inst.getValues = inst.getVal;

        inst.getValue = getVal;
        // ---

        // ---

        return {
            width: 50,
            layout: layout,
            headerText: false,
            anchor: input,
            formatResult: function (d) {
                return d[optionWheelIdx];
            },
            parseValue: function (val) {
                getOption(val === undefined ? elm.val() : val);

                return groupWheel ? [group, option] : [option];
            },
            onValueTap: onTap,
            onValueFill: onFill,
            onBeforeShow: function () {
                if (multiple && s.counter) {
                    s.headerText = function () {
                        var length = 0;
                        $.each(selectedValues, function () {
                            length++;
                        });
                        return length + ' ' + s.selectedText;
                    };
                }

                // In header text replace {value} with the option text instead of the value
                if (isString(s.headerText) && /{value}/.test(s.headerText)) {
                    headerText = s.headerText;
                    s.headerText = function (v) {
                        return headerText.replace(/\{value\}/i, options[v].text);
                    };
                }

                getOption(elm.val());

                if (groupWheel) {
                    prevGroup = group;
                    inst._tempWheelArray = [group, option];
                }

                prepareData();

                s.wheels = genWheels();
            },
            onMarkupReady: function (dw) {
                dw.addClass('dw-select');

                $('.dwwl' + groupWheelIdx, dw).on('mousedown touchstart', function () {
                    clearTimeout(timer);
                });

                if (groupHdr) {
                    $('.dw', dw).addClass('dw-select-gr');
                    $('.dw-li[data-val^="__group"]', dw).addClass('dw-w-gr');
                }

                if (multiple) {
                    dw.addClass('dwms');

                    $('.dwwl', dw).on('keydown', function (e) {
                        if (e.keyCode == 32) { // Space
                            e.preventDefault();
                            e.stopPropagation();
                            onTap($('.dw-sel', this));
                        }
                    }).eq(optionWheelIdx).addClass('dwwms').attr('aria-multiselectable', 'true');

                    origValues = $.extend({}, selectedValues);
                }
            },
            validate: function (dw, i, time) {
                var j,
                    v,
                    temp = inst.getArrayVal(true),
                    t = $('.dw-ul', dw).eq(optionWheelIdx);

                if (i === undefined && multiple) {
                    v = selectedValues;
                    j = 0;

                    $('.dwwl' + optionWheelIdx + ' .dw-li', dw).removeClass(selectedClass).removeAttr('aria-selected');

                    for (j in v) {
                        $('.dwwl' + optionWheelIdx + ' .dw-li[data-val="' + v[j] + '"]', dw).addClass(selectedClass).attr('aria-selected', 'true');
                    }
                }

                if (groupWheel && (i === undefined || i === groupWheelIdx)) {
                    group = +temp[groupWheelIdx];
                    if (group !== prevGroup) {
                        option = groups[group].options[0].value;
                        s.wheels = genWheels();
                        if (!change) {
                            inst._tempWheelArray = [group, option];
                            s.readonly = [false, true];
                            clearTimeout(timer);
                            timer = setTimeout(function () {
                                change = true;
                                prevGroup = group;
                                inst.changeWheel([optionWheelIdx], undefined, true);
                                s.readonly = origReadOnly;
                            }, time * 1000);
                            return false;
                        }
                    } else {
                        s.readonly = origReadOnly;
                    }
                } else {
                    option = temp[optionWheelIdx];
                }

                $.each(s.invalid, function (i, v) {
                    $('.dw-li[data-val="' + v + '"]', t).removeClass('dw-v');
                });

                change = false;
            },
            onClear: function (dw) {
                selectedValues = {};
                input.val('');
                $('.dwwl' + optionWheelIdx + ' .dw-li', dw).removeClass(selectedClass).removeAttr('aria-selected');
            },
            onCancel: function () {
                if (!inst.live && multiple) {
                    selectedValues = $.extend({}, origValues);
                }
            },
            onDestroy: function () {
                input.remove();
                elm.removeClass('dw-hsel').removeAttr('tabindex');
            }
        };
    };

})(jQuery);
