var mad = mad || {};

mad.appTelemetry = (function () {
    var isAppRunningInSimulator;

    var SEVERITY = {
        Verbose:  "Verbose",
        Information: "Information",
        Warning:  "Warning" ,
        Error: "Error",
        Critical: "Critical",
    };


    /**
     * Initializes azure app insights
     * @returns {} 
     */
    function initAppInsights() {
        if (device && appInsights && appInsights.context && appInsights.context.device) {
            appInsights.context.device.osversion = device.version;
            appInsights.context.device.os = device.platform;
            appInsights.context.device.model = device.model;
            appInsights.context.device.id = device.uuid;
        }
    }

    /**
     * Sets current app version in AzureAppInsights
     * @param ver current app version
     * @returns void
     */
    function setAppVersionForAzureAppInsights(ver) {
        if (device && appInsights && appInsights.context && appInsights.context.application) {
            appInsights.context.application.ver = ver;
        }
    }

    /**
     * Adds madbarz userId to Azure Appinsights context
     * @param id This is Madbarz userId  
     */
    function setUserIdForAzureAppInsights(id) {
        if (!isAppRunningInSimulator() && appInsights) {
            appInsights.setAuthenticatedUserContext(id);
        }
    }

    /**
    * Removes madbarz userId from Azure Appinsights context
    */
    function removeUserIdFromAzureAppInsights() {
        if (!isAppRunningInSimulator() && appInsights) {
            appInsights.clearAuthenticatedUserContext();
        }
    }

    /**
    * Adds tracks event in Azure App insights
    * @param id This is Madbarz userId  
    */
    function trackEvent(eventName, extraData) {
        if (!isAppRunningInSimulator() && appInsights) {
            appInsights.trackEvent(eventName, extraData);

        }
    }

     /**
     *  Logs exceptions
     * @param error 
     * @param context Context where info was logged
     * @param additionalInfo Some additional Message
     * @param severity Severity level
     * @param userId UserId
     * @param userNameOrEmail username or email
     */
    function trackException(error, context, additionalInfo, severity, userId, userNameOrEmail ) {
        if (!isAppRunningInSimulator() && appInsights) {
            context = (typeof context !== 'undefined') ? context : "Context unknown";
            additionalInfo = (typeof additionalInfo !== 'undefined') ? additionalInfo : "";
            severity = (typeof severity !== 'undefined') ? severity : SEVERITY.Information;
            userId = (typeof userId !== 'undefined') ? userId : "";
            userNameOrEmail = (typeof userNameOrEmail !== 'undefined') ? userNameOrEmail : "";


            var extraData = {
                "AdditionalInfo": additionalInfo,
                "Context": context,
                "AppVersion": version,
                "ConnectionType": navigator.connection.type.toString(),
                "Severity": severity,
                "UserId": userId,
                "UserNameOrEmail": userNameOrEmail
            };
            appInsights.trackException(error, extraData);
        }
    }

    /**
     *  Logs diagnostic info
     * @param message 
     * @param context Context where info was logged
     * @param additionalInfo Some additional Message
     * @param severity Severity level
     * @param userId UserId
     * @param userNameOrEmail username or email
     */
    function logInfo(message, context, additionalInfo, severity, userId, userNameOrEmail) {

        context = (typeof context !== 'undefined') ? context : "Context unknown";
        additionalInfo = (typeof additionalInfo !== 'undefined') ? additionalInfo : "";
        severity = (typeof severity !== 'undefined') ? severity : SEVERITY.Information;
        userId = (typeof userId !== 'undefined') ? userId : "";
        userNameOrEmail = (typeof userNameOrEmail !== 'undefined') ? userNameOrEmail : "";

       
        if (!isAppRunningInSimulator() && appInsights) {
            var extraData = {
                "AdditionalInfo": additionalInfo,
                "Context": context,
                "AppVersion": version,
                "ConnectionType": navigator.connection.type.toString(),
                "Severity": severity,
                "UserId": userId,
                "UserNameOrEmail": userNameOrEmail
            };
            appInsights.trackTrace(message, extraData);
        }
    }

    /**
     * 
     * @param {} name 
     * @param {} url 
     * @param {} extraData 
     * @param {} measurements 
     * @returns {} 
     */
    function trackPageView(name, url, extraData, measurements) {
        if (!isAppRunningInSimulator() && appInsights) {
            appInsights.trackPageView(name, url, extraData, measurements);

        }
    }


    isAppRunningInSimulator = function () {
        return window.navigator.simulator === true;
    };

    return {
        initAzureAppInsights: initAppInsights,
        setAppVersionForAzureAppInsights :  setAppVersionForAzureAppInsights,
        setUserIdForAzureAppInsights: setUserIdForAzureAppInsights,
        removeUserIdFromAzureAppInsights: removeUserIdFromAzureAppInsights,
        trackEvent: trackEvent,
        trackException: trackException,
        logInfo: logInfo,
        trackPageView: trackPageView,
        SEVERITY: SEVERITY

    };

}());