/* primjer:

 var chart = d3.dr.graf1()
 .x(function(d) { return formatDate.parse(d.date); })
 .y(function(d) { return +d.price; });

 var formatDate = d3.time.format("%b %Y");

 d3.csv("sp500.csv", function(data) {
 d3.select("#example")
 .datum(data)
 .call(chart);
 });

 */

// CREATE CUSTOM SUB-SELECTIONS
d3.selection.prototype.first = function() {
	return d3.select(this[0][0]);
};
d3.selection.prototype.last = function() {
	var last = this.size() - 1;
	return d3.select(this[0][last]);
};

d3.dr = {};

d3.dr.graf1 = function () {

	// -- default values ----------------------------------
	var margin = {top: 30, right: 0, bottom: 15, left: 0},
		yOffset = {top: 60, bottom: 20},                        //additional y offset for graph
		width = $(window).width(),
		height = $(window).height() * 0.55,
		graphWidth = width - margin.left - margin.right,
		graphHeight = height - margin.top - margin.bottom - yOffset.bottom;

	var type = "exercises",
		info = {},
		granularity = 2;        //sve

	var actualSelection;

	var imageSize = 90;

	var dan = d3.time.format("%d");                 // day from date
	var ddmmgg = d3.time.format("%d.%m.%Y");
	var danMje7 = d3.time.format("%a. %d %b");
	var danMje30 = d3.time.format("%d %b");
	var danMjeAll = d3.time.format("%b %e, %Y");

	var xValue = function (d, i) {
			return i;
		},
		xDate = function (d) {
			return d.Date;
		},
		yValue = function (d) {
			return d.Value;
		};

	// Set the scale & ranges
	var xScale = d3.scale.linear().range([0, graphWidth]),
		yScale = d3.scale.linear().range([graphHeight, 0 + yOffset.top]);

	// scaled value functions
	function xValueScaled(d, i) {
		return xScale(i);
	}            // Ith value for x scaled
	function yValueScaled(d) {
		return yScale(d.Value);
	}           // 2nd value for y scaled

	// Define the axes
	var xAxis = d3.svg.axis().scale(xScale).orient("bottom"),
		yAxis = d3.svg.axis().scale(yScale).orient("left");

	var yTicks = 4;         // moze se dodati postavljenje pomocu metode

	/*var gradientData,
		data1 = [           // data for exercises & scores gradient
			{offset: "0%", color: "red"},
			{offset: "40%", color: "red"},
			{offset: "45%", color: "goldenrod"},
			{offset: "50%", color: "goldenrod"},
			{offset: "55%", color: "forestgreen"},
			{offset: "100%", color: "darkgreen"}
		],
		data2 = [           // data for weight gradient
			{offset: "0%", color: "red"},
			{offset: "25%", color: "red"},
			{offset: "30%", color: "darkgreen"},
			{offset: "40%", color: "darkgreen"},
			{offset: "45%", color: "red"},
			{offset: "100%", color: "red"}
		];
	*/
	
	var gradientData,
		data1 = [           // data for exercises & scores gradient
			{offset: "0%", color: "#a42901"},
			{offset: "100%", color: "#ffeb41"}
		],
		data2 = [           // data for weight gradient
			{offset: "0%", color: "#a42901"},
			{offset: "100%", color: "#ffeb41"}
		];


	// define graph functions
	var lineFunction = d3.svg.line().x(xValueScaled).y(yValueScaled).interpolate("monotone");
	var lineFunction2 = d3.svg.line().x(xValueScaled).y(yValueScaled).interpolate("cardinal");
	var lineFunction2 = d3.svg.line().x(xValueScaled).y(yValueScaled).interpolate("basis");
	var lineTestFunction = d3.svg.line().x(xValueScaled).y(yValueScaled);

	var areaFunction = d3.svg.area()
		.x(xValueScaled)
		.y0(height - margin.top - margin.bottom - yOffset.bottom)
		.y1(yValueScaled)
		.interpolate("basis");

	function recalculate() {
		console.log("recalculateing");
		graphWidth = width - margin.left - margin.right;
		graphHeight = height - margin.top - margin.bottom - yOffset.bottom;

		xScale.range([0, graphWidth]),
		yScale.range([graphHeight, 0 + yOffset.top]);

	}

	function stepY(min, max) {
		var step = Math.round((max - min) / yTicks);
		return step < 1 ? .5 : step;
	}

	function chart(selection) {

		chart.selection(selection);

		selection.each(function (data) {

			//console.log("graph data", data);

			// set domains
			xScale.domain([0, data.length - 1]);

			/* normal min i max Y values */
			var yMin = d3.min(data, yValue);
			var yMax = d3.max(data, yValue);

			yMin -= 2;
			yMax += 2;

			if (yMin < 0 ) {yMin = 0};

			yScale.domain([yMin, yMax]);

			d3.selectAll("svg").remove();

			// Select the svg element, if it exists.
			var svg = d3.select(this).selectAll("svg").data([data]);

			// Otherwise, create the skeletal chart.
			var gEnter = svg.enter().append("svg").append("g");
			var gExit = svg.exit();

			gEnter.append("g").attr("class", "x axis");
			gEnter.append("g").attr("class", "y axis");
			//gEnter.append("image").attr("class", "icon");
			//gEnter.append("path").attr("class", "area");
			gEnter.append("text").attr("class", "graf-poruka");
			gEnter.append("text").attr("class", "jmj");

			if (type == "exercises" || type == "scores") {
				gradientData = data1
			} else {
				gradientData = data2
			}

				var grad = gEnter.append("linearGradient")
				.attr({"id": "line-gradient", "gradientUnits": "userSpaceOnUse",
					"x1": "0%", "y1": "100%",
					"x2": "0%", "y2": "0%"})
				.selectAll("stop")
				.data(gradientData);

			grad.enter().append("stop")
					.attr("offset", function(d) { return d.offset; })
					.attr("stop-color", function(d) { return d.color; });

			gEnter.append("svg:path").attr("class", "line");
			//gEnter.append("svg:path").attr("class", "line2");
			//gEnter.append("svg:path").attr("class", "lineTest");

			// Update the outer dimensions.
			svg.attr("width", width).attr("height", height);

			// Update the inner dimensions.
			var g = svg.select("g")
				.attr("transform", "translate(" + margin.left + "," + margin.top + ")")
				.transition().duration(111).ease("sin-in-out");

			var brPodataka = data.length - 1;
			var brTikova;
			
			if (granularity == 0) {
				brTikova = (brPodataka < 6) ? brPodataka : 6;
				xAxis.ticks(brTikova).tickFormat(
					function (v, i) {
						var dat = danMje7(data[v].Date);
						return dat;
					})
			} else if (granularity == 1) {
				brTikova = (brPodataka < 6) ? brPodataka : 6;
				xAxis.ticks(brTikova).tickFormat(
					function (v, i) {
						var dat = danMje30(data[v].Date);
						return dat;
					})
			} else {
				brTikova = (brPodataka < 5) ? brPodataka : 5;
				xAxis.ticks(brTikova).tickFormat(
					function (v, i) {
						var dat = danMjeAll(data[v].Date);
						return dat;
					})
			}

			var poruka = g.select(".graf-poruka")
				.attr("x", width * .5)
				.attr("y", height * .5)
				.style("text-anchor", "middle");

			if (data.length < 2){
				poruka.text("not enough data");
			}else{
				poruka.text("");
			}

			var jmj = g.select(".jmj")
				.attr("x", width * .5)
				.attr("y", 10)
				.style("text-anchor", "middle");

			if (type == "exercises") {
				jmj.text(info.secReps);
			} else if (type == "scores") {
				jmj.text("Points");
			} else {                      // kg/lbs
				jmj.text(type);
			}

			/* pozicioniranje x osi i poravnanje labela na x osi 1. left, zadnja right */
			var stvarniBrojTikova = 0
			var xOs = g.select(".x.axis");

			xOs.call(xAxis)
				.selectAll("text")
				.each(
					function (v, i) {
						stvarniBrojTikova += 1;
					});

			xOs.call(xAxis)
				.attr("transform", "translate(0," + yScale.range()[0] + ")")
				.selectAll("text")
					.style("text-anchor", function(d, i) {
						if (i==0){
						   return "start";
						}else if (i==stvarniBrojTikova-1){
							return "end";
						}else{
							return "middle";
						}
					});

			//for 7 days move every even label down
			var allXlabs = xOs.selectAll("text");               //zato sto je zadnji prethodno popapan
			allXlabs.transition()
				//.duration(300)
				//.ease("exp-out")
				.attr("transform", function(d, i) {

				var pomak = 0;
				if (granularity == 0 || granularity == 2){
				   pomak = (i % 2 != 0) ? 14 : 0;
				}

				return "translate(" + 0 + "," + pomak + ")"; });

			yAxis.tickSize(-width, 0, 0)
				.tickValues(d3.range(yMin, yMin + stepY(yMin, yMax) * yTicks + 1, stepY(yMin, yMax)))
				//.tickPadding(-65)
				.tickPadding(-width * .5)
				.tickFormat(
				function (v, i) {
					/*
					if (type == "exercises") {
						return v + " " + info.secsReps;
					} else if (type == "scores") {
						return v + " Points";
					} else {                      // kg/lbs
						return v.toFixed() + ' ' + type;
					}
					*/

					if (type != "exercises" && type != "scores"){        // for kg/lbs
						return v.toFixed(2);
					}else{
						return v;
					}
				});

			g.select(".y.axis")
				.duration(700)
				.style("opacity", "0.4")
				.call(yAxis);

			// nakon sto se nacrta y os pomakni labele
			var yOs = g.select(".y.axis");
			var allYlabs = yOs.selectAll("text");
			allYlabs.attr("dy", "-0.2em").style("text-anchor", "middle");

			// update icon
			/*
			if (type == "exercises") {
				var imgLnk = info.selectedImage;
				if (imgLnk === null) {
					imgLnk = 'images/exercise/noExercise.png';
					g.select(".icon")
						.duration(750)
						.attr("xlink:href", imgLnk)
						.style("opacity", "0.5")
						.attr("x", width  -  20 - imageSize)
						.attr("y", height - 80 - imageSize)
						.attr("width", imageSize)
						.attr("height", imageSize);
				}else{
					slashPoz = imgLnk.lastIndexOf('/');
					var imgName =  imgLnk.substring(slashPoz + 1);
					imgLnk = 'images/exercise/' + imgName;
					$.ajax({
						url: imgLnk,
						success: function(data){
							g.select(".icon")
								.duration(750)
								.attr("xlink:href", imgLnk)
								.style("opacity", "0.5")
								.attr("x", width  -  20 - imageSize)
								.attr("y", height - 80 - imageSize)
								.attr("width", imageSize)
								.attr("height", imageSize);
						},
						error: function(data){
							g.select(".icon")
								.duration(750)
								.attr("xlink:href", 'images/exercise/noExercise.png')
								.style("opacity", "0.5")
								.attr("x", width  -  20 - imageSize)
								.attr("y", height - 80 - imageSize)
								.attr("width", imageSize)
								.attr("height", imageSize);
						}
					})
				}
			}else{
				g.select(".icon")
					.transition()
						.duration(400)
						.style("opacity", "0")
						.attr("xlink:href", "");
			}
			*/

			// Update the area path.
			/*
			g.select(".area")
				.duration(500)
				.attr("d", areaFunction.y0(yScale.range()[0]));
			*/

			// Update the line path.
			 g.select(".line")
				 .duration(500)
				 .attr("d", lineFunction);

			/*
			if (granularity == 0) {             //7
				g.select(".line").classed({'thinner': false, 'thin': false})
			} else if (granularity == 1) {      //30
				g.select(".line").classed({'thinner': true, 'thin': false})
			}else{                              //all
				g.select(".line").classed({'thinner': false, 'thin': true})
			}
			*/

			g.select(".line2")
				.duration(500)
				.attr("d", lineFunction2);
			g.select(".lineTest")
				.duration(500)
				.attr("d", lineTestFunction);
/*
			gExit.transition()
				.duration(500)
				.remove();
*/
		});
	}

	chart.type = function (_) {
		type = _;
		return this;
	};

	chart.info = function (_) {
		if (!arguments.length) return info;
		info = _;
		return this;
	};

	chart.granularity = function (_) {
		if (!arguments.length) return granularity;
		granularity = _;

		var g = d3.select("g");
		//console.log("g je", g);

		if (granularity == 0) {             //7
			g.select(".line").classed({'thinner': false, 'thin': false})
		} else if (granularity == 1) {      //30
			g.select(".line").classed({'thinner': true, 'thin': false})
		}else{                              //all
			g.select(".line").classed({'thinner': false, 'thin': true})
		}

		return this;
	};

	chart.margin = function (_) {
		if (!arguments.length) return margin;
		margin = _;
		recalculate();
		return this;
	};

	chart.width = function (_) {
		if (!arguments.length) return width;
		width = _;
		recalculate();
		return this;
	};

	chart.height = function (_) {
		if (!arguments.length) return height;
		height = _;
		recalculate();
		return this;
	};

	chart.x = function (_) {
		if (!arguments.length) return xValue;
		xValue = _;
		recalculate();
		return this;
	};

	chart.y = function (_) {
		if (!arguments.length) return yValue;
		yValue = _;
		recalculate();
		return this;
	};

	chart.selection = function (_) {
		if (!arguments.length) return actualSelection;
		actualSelection = _;
		return this;
	}
	return chart;

}
