var mad = mad || {};

mad.installation = (function () {

    var resourceUrl = mad.common.variables.serverUrl + "api/AppInstallations/";
    //var appVersion = mad.common.version;
    var appVersion = version;       // glob var postavlena u commonu


    var getInstallationId = function () {
        var installationId = mad.local.loadVariable("installationId");
        if (installationId === "")
            return null;
        return installationId;
    };
    
    var getPnsHandle = function() {
        var pnsHandle = mad.local.loadVariable("pnsHandle");
        if (pnsHandle === "")
            return null;
        return pnsHandle;
    };

    var isRegistered = function () {
        return getInstallationId() !== null;
    };

    var getVersion = function() {
        return appVersion;
    };

    var getDefaultNotificationSubscriptions = function () {
        return [
            { type: "TreningRespect", uiName: "respects", subscribed: true },
            { type: "TreningComment", uiName: "comments", subscribed: true },
            { type: "TreningSave", uiName: "favorites", subscribed: true },
            { type: "Follows", uiName: "follows", subscribed: true },
            { type: "Trending", uiName: "trending", subscribed: true }
        ];
    };

    var loadNotificationSubscriptions = function () {
        var setting = mad.local.loadVariable("subscribedNotificationTypes");
        if (setting === null || setting === "")
            return null;
        var notificationSubscriptions = JSON.parse(setting);
        if (notificationSubscriptions === "")
            return null;
        return notificationSubscriptions;
    };

    var saveNotificationSubscriptions = function(notificationSubscriptions) {
        mad.local.saveVariable("subscribedNotificationTypes", JSON.stringify(notificationSubscriptions));
    };

    var getNotificationSubscriptions = function () {
        var subscriptions = loadNotificationSubscriptions();
        if (subscriptions === null)             // If there are no preferences set for notification types the users is subscribed to, create default preferences.
            return getDefaultNotificationSubscriptions();
        return subscriptions;
    };

    var isSubscribedToNotifications = function (uiName) {
        var subscriptions = getNotificationSubscriptions();
        for (var i = 0; i < subscriptions.length; i++)
            if (subscriptions[i].uiName === uiName)
                return subscriptions[i].subscribed;
        return false;
    };

    var clearNotificationSubscriptions = function () {
        var subscriptions = loadNotificationSubscriptions();
        if (subscriptions === null)                         // If there are no preferences set for notification types the users is subscribed to, create default preferences.
            subscriptions = getDefaultNotificationSubscriptions();
        for (var i = 0; i < subscriptions.length; i++)      // Set all notification type subscriptions to unsubscribed
            subscriptions[i].subscribed = false;
        saveNotificationSubscriptions(subscriptions);
    };

    var getSubscriptionTypes = function(subscriptions) {
        var types = [];
        for (var i = 0; i < subscriptions.length; i++)
            if (subscriptions[i].subscribed)
                types.push(subscriptions[i].type);
        return types;
    }

    var register = function (handle, ver, onSuccess, onError) {
        var installationId = getInstallationId();
        var installationModel = {
            InstallationId: installationId,
            PlatformCode: getPlatformCode(),
            PnsHandle: handle,
            Version: version,
            DeviceInfo: getDeviceInfo(),
            NotificationTemplates: getNotificationTemplates(),
            NotificationSubscriptions: getSubscriptionTypes(getNotificationSubscriptions())
        };

        if (installationId === null) {
            mad.data.post({ putanja: resourceUrl, akcija: "POST" }, installationModel, false)
                .done(function (data, textStatus, jqXHR) {
                    setInstallationId(data);
                    setPnsHandle(handle);
                    console.log("mad.installation.register", "Registered installation", data);
                })
                .fail(function (jqXHR, textStatus, errorThrown) {
                    console.log("mad.installation.register", textStatus, errorThrown);
                });
        } else {
            mad.data.put(resourceUrl + installationId, installationModel, false)
                .done(function(data, textStatus, jqXHR) {
                    setPnsHandle(handle);
                    console.log("mad.installation.register", "Registration updated", installationId);
                })
                .fail(function(jqXHR, textStatus, errorThrown) {
                    console.log("mad.installation.register", textStatus, errorThrown);
                });
        }
    };

    var unregister = function (onSuccess, onError) {
        var installationId = getInstallationId();

        if (installationId !== null) {
            mad.data.delete(resourceUrl + installationId, null, false)
                .done(function(data, textStatus, jqXHR) {
                    console.log("mad.installation.register", "Registration deleted", installationId);
                    setInstallationId(null);
                })
                .fail(function(jqHXR, textStatus, errorThrown) {
                    console.log("mad.installation.register", textStatus, errorThrown);
                });
        }
    };

    var setNotificationSubscription = function (uiName, subscribe) {
        var subscriptions = getNotificationSubscriptions();

        for (var i = 0; i < subscriptions.length; i++)
            if (subscriptions[i].uiName === uiName) {
                subscriptions[i].subscribed = subscribe;
                break;
            }

        var pnsHandle = mad.installation.getPnsHandle();
        saveNotificationSubscriptions(subscriptions);
        register(pnsHandle, mad.common.version);
    };

    var associateUser = function(onSuccess, onError) {
        var pnsHandle = mad.installation.getPnsHandle();
        var notificationSubscriptions = getDefaultNotificationSubscriptions();      // Set default notification preferences when a user logs in.
        saveNotificationSubscriptions(notificationSubscriptions);
        register(pnsHandle, mad.common.version);
    };

    var disassociateUser = function(onSucces, onError) {
        var pnsHandle = mad.installation.getPnsHandle();
        clearNotificationSubscriptions();                                           // Set notification preferences to all notification types unsubscribed.
        register(pnsHandle, mad.common.version);
    };


    function getPlatformCode() {
        switch (device.platform) {
            case "android":
            case "Android":
                return "gcm";
            case "iOS":
                return "apns";
            default:
                return undefined;
        }
    }

    function setInstallationId(installationId) {
        mad.local.saveVariable("installationId", installationId);
    }
    
    function setPnsHandle(handle) {
        mad.local.saveVariable("pnsHandle", handle);
    }

    function getDeviceInfo() {
        return [
            { Name: "Name", Value: device.name },
            { Name: "Platform", Value: device.platform },
            //{ Name: "UUID", Value: device.uuid },                 // TODO: Research if this UUID is possible to obtain and use to identify a device!
            { Name: "OSVersion", Value: device.version },
            { Name: "Model", Value: device.model }
        ];
    }

    function getNotificationTemplates() {
        switch (device.platform) {
            case "android":
            case "Android":
                // TODO: Refine the Android push notification template!
                return [
                    { Name: "default", Template: '{ "data": { "message": "$(messageBody)", "title": "$(messageTitle)", "notId": "#(categoryId)", "notification": { "id": "$(id)", "title": "$(title)", "text": "$(text)", "type": "$(type)", "link": "$(link)", "fromUserId": "$(fromUserId)", "toUserId": "$(toUserId)" } } }' }
                ];
            case "iOS":
                // TODO: Refine the iOS push notification templates!
                return [
                    { Name: "dafault", Template: '{ "aps": { "alert": { "title": "$(messageTitle)", "body": "$(messageBody)" }, "sound": "notifMain.wav", "badge": "#(unreadCount)" }, "notification": { "id": "$(id)", "type": "$(type)", "fromUserId": "$(fromUserId)", "link": "$(link)" } }' }
                ];
            default:
                return [];
        }
    }

    function checkIsUpgrade() {
        // Check if the notification settings need to be upgraded
        var defaultSubscriptions = getDefaultNotificationSubscriptions();
        var currentSubscriptions = loadNotificationSubscriptions();

        if (currentSubscriptions === null) {
            saveNotificationSubscriptions(getDefaultNotificationSubscriptions());
            return;
        }

        // Check if all subscription types are present in the currently stored subscriptions in the local storage
        var hasMissing = false;
        for (var i = 0; i < defaultSubscriptions.length; i++) {
            var isFound = false;
            for (var j = 0; j < currentSubscriptions.length; j++)
                if (defaultSubscriptions[i].type === currentSubscriptions[j].type) {
                    isFound = true;
                    break;
                }
            if (!isFound) {
                hasMissing = true;
                break;
            }
        }

        if (hasMissing)
            saveNotificationSubscriptions(getDefaultNotificationSubscriptions());
    }

    return {
        isRegistered: isRegistered,
        register: register,
        unregister: unregister,
        associateUser: associateUser,
        disassociateUser: disassociateUser,
        getInstallationId: getInstallationId,
        getPnsHandle: getPnsHandle,
        getVersion: getVersion,
        getNotificationSubscriptions: getNotificationSubscriptions,
        subscribeToNotifications: setNotificationSubscription,
        isSubscribedToNotifications: isSubscribedToNotifications,
        checkIsUpgrade: checkIsUpgrade
    };

}());