var mad = mad || {};

/**
 * Native storage should only be used for storing small amounts of data as it stores data in
 * iOS (NSUserDefaults) & Android SharedPreferences
 * This is wrapper for plugin which can be found here: https://github.com/TheCocoaProject/cordova-plugin-nativestorage
*/

mad.nativeStorage = (function () {

    var isAppRunningInSimulator;

    /**
     * @summary Saves data to native storage, note supports any type which can be stringified
     * @param {string} key Unique key to which object data is saved
     * @param {any} obj Object to save
     * @param {function()} successCallback Function to execute on successful save
     * @param {function()} errorCallback Function to execute on error while saving
     */
    function setItem(key, obj, successCallback, errorCallback) {

        //key must be string
        if (typeof key === "string") {
            //if plug-ins are initialized
            if (window && window.plugins && !isAppRunningInSimulator()) {
                var nativeStorage = window.NativeStorage;
                nativeStorage.setItem(key, obj, successCallback, errorCallback);
            } 
        } 
    }

    /**
     * @summary Gets data from native storage
     * @param {string} key Unique key for object we are retrieving 
     * @param {function()} successCallback Function to call if object was retrieved successfully
     * @param {function()} errorCallback Function to call if object could not be retrieved
     */
    function getItem(key, successCallback, errorCallback) {

        //key must be string
        if (typeof key === "string") {
            //if plug-ins are initialized
            if (window && window.plugins && !isAppRunningInSimulator()) {
                var nativeStorage = window.NativeStorage;
                nativeStorage.getItem(key, successCallback, errorCallback);
            }
        }

       
    }

    /**
     * @summary Deletes item at given key
     * @param {string} key Key for item to be deleted
      * @param {function()} successCallback Function to call if delete was success
     * @param {function()} errorCallback Function to call if delete failed
     */
    function removeItem(key, successCallback, errorCallback) {
        //key must be string
        if (typeof key === "string") {
            //if plug-ins are initialized
            if (window && window.plugins && !isAppRunningInSimulator()) {
                var nativeStorage = window.NativeStorage;
                nativeStorage.remove(key, successCallback, errorCallback);
            }
        }


    }

    /**
     * @summary Deletes all data stored in native storage
     * @param {function()} successCallback Function to call if delete was success
     * @param {function()} errorCallback Function to call if delete failed
     */
    function removeAll(successCallback, errorCallback) {
        //if plug-ins are initialized
        if (window && window.plugins && !isAppRunningInSimulator()) {
            var nativeStorage = window.NativeStorage;
            nativeStorage.clear(successCallback, errorCallback);
        }
    }

    isAppRunningInSimulator = function () {
        return window.navigator.simulator === true;
    };

    return {
        setItem: setItem,
        getItem: getItem,
        removeItem: removeItem,
        removeAll: removeAll
    };

}());