var mad = mad || {};

mad.subscriptionService = (function() {
  /*=============================================================================
    * 
    * **************************** OTHER ****************************
    * 
    *=============================================================================*/

  var trackPurchase = function(subscriptionId) {
    var revenue = 0;
    var subsName = "";
    var appleSubscriptionIds = [
      "com.madbarz.madbarzapp.premiumsubs",
      "com.madbarz.madbarzapp.premiumsubs.3m",
      "com.madbarz.madbarzapp.premiumsubs.6m",
      "com.madbarz.madbarzapp.premiumsubs.12m"
    ];
    var pricesProceedsiOs = [7.8, 23.11, 41.86, 55.61];
    var possibleSubscriptions = [
      "One Month",
      "Three Months",
      "Six Months",
      "Twelve Months"
    ];

    if (device.platform === "Android") {
      //price = viewModel.get("pricesProceedsAndroid")[gumb];
    } else {
      if (subscriptionId) {
        var index = appleSubscriptionIds.indexOf(subscriptionId);
        if (index >= 0) {
          subsName = possibleSubscriptions[index];
          revenue = pricesProceedsiOs[index];
          mad.appsFlyer.trackPurchase("Premium - " + subsName, revenue);
        }
      }
     
    }
  };

  /**
   * @summary This method gets all subscription product ids from local storage
   * @returns {Object} Returns Object with props: Apple, Google, Web (these are lists of the product ids for each platform)
   */
  var getSubscriptionProductIdsFromLocalStorage = function() {
    var dataFromLocalStorage = mad.local.loadVariableParse(
      "subscriptionProductIds"
    );

    //if data wasn't in local storage empty array is returned
    if (dataFromLocalStorage.length < 1) {
      return {
        Apple: [],
        Google: [],
        Web: []
      };
    }

    return dataFromLocalStorage;
  };

  /**
   * @summary This method gets all subscription product ids
   * @returns {Object} Returns Object with props: Apple, Google, Web (these are lists of the product ids for each platform)
   */
  var getSubscriptionProductIds = function() {
    if (mad.common.variables.subscriptionProductIds) {
      if (
        mad.common.variables.subscriptionProductIds.Apple.length > 0 &&
        mad.common.variables.subscriptionProductIds.Google.length > 0
      ) {
        return mad.common.variables.subscriptionProductIds;
      } else {
        return getSubscriptionProductIdsFromLocalStorage();
      }
    } else {
      return getSubscriptionProductIdsFromLocalStorage();
    }
  };

  /*=============================================================================
    * 
    * ****************** DETERMINING IF USER HAS ACCESS TO PRO *******************
    * 
    *=============================================================================*/

  /**
   *  This method updates common variable with information whether user has PRO or not (the one which was initially set to -1)
   * @param {boolean} hasPro True / False - if user has pro or not
   * @param {any} deferred Promise to resolve
   */
  var updateCommonVarHasUserAccesToPro = function(hasPro, deferred) {
    if (typeof hasPro === "boolean") {
      if (hasPro) {
        mad.common.variables.HasUserAccesToPro = 1;
        deferred.resolve(true);
      } else {
        mad.common.variables.HasUserAccesToPro = 0;
        deferred.resolve(false);
      }
    } else {
      deferred.resolve(false);
    }
  };

  /**
   * @summary This methods adds record of subscription on server
   * @param {string} productCode Product (code) for which subscription  was purchased (or renewed)
   * @param {Object} receipt Apple's receipt containing user's in-app purchases
   * @param {any} deferred Promise to resolve
   */
  var recordSubscriptionOnServerForApple = function(
    productCode,
    receipt,
    deferred
  ) {
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/PurchaseSubscriptionOnIos",
      akcija: "POST"
    };

    var options = {
      Data: receipt,
      ProductCode: productCode
    };

    mad.data
      .load(route, options, false, true)
      .done(function(data) {
        console.log("Data from buying subscription on server is:", data);
        if (data == 0) {
          mad.common.hideLoader();

          if (deferred) {
            updateCommonVarHasUserAccesToPro(true, deferred);
          }
        } else {
          mad.common.hideLoader();

          if (deferred) {
            updateCommonVarHasUserAccesToPro(false, deferred);
          }
        }
      })
      .fail(function(data) {
        console.log(data);
        mad.common.hideLoader();
        if (deferred) {
          //even if for some reason recording purchase on server fails ... user should still get access to pro
          updateCommonVarHasUserAccesToPro(true, deferred);
        }
      });
  };

  /**
   * @summary This method adds subscription record on server when updating subscription without whole receipt data (i.e. when receipt is already parsed and validated on mobile)
   * @param {string} productCode Product (code) for which subscription  was purchased (or renewed)
   * @param {any} deferred Promise to resolve
   * @param {Object} purchaseData Data containing purchase info (such as purchase date, expiry date etc.)
   */
  var recordSubscriptionUpdateOnServerForApple = function(
    productCode,
    deferred,
    purchaseData
  ) {
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/UpdateiOsSubscriptionWithoutReceipt",
      akcija: "POST"
    };

    var options = {
      ProductCode: productCode,
      AppleStoreProductId: purchaseData.productId,
      PurchaseDateUtc: purchaseData.purchaseDate,
      ExpiresDateUtc: purchaseData.subscriptionExpirationDate,
      TransactionId: purchaseData.transactionIdentifier
    };

    mad.data
      .load(route, options, false, true)
      .done(function(data) {
        //subscription update was successful
        updateCommonVarHasUserAccesToPro(true, deferred);
      })
      .fail(function(data) {
        console.log(data);
        if (deferred) {
          //even if for some reason recording purchase on server fails ... user should still get access to pro
          updateCommonVarHasUserAccesToPro(true, deferred);
        }
      });
  };

  /**
   * @summary This method refreshes Apple receipt locally, and checks if user has active subscription for the product or not
   * @param {string} productCode Product (code) for which we are checking subscription status
   * @param {any} deferred Promise to resolve
   */
  var refreshAppleReceiptAndValidateIt = function(productCode, deferred) {
    if (inAppPurchase) {
      inAppPurchase
        .getReceipt()
        .then(function(receiptData) {
          inAppPurchase
            .getReceiptBundle()
            .then(function(receipt) {
              //If receipt doesn't exist, user doesn't have access to Pro
              if (!receipt) {
                updateCommonVarHasUserAccesToPro(false, deferred);
                return;
              }

              // check are there any in-app purchases
              if (receipt.inAppPurchases.length === 0) {
                updateCommonVarHasUserAccesToPro(false, deferred);
                return;
              }

              var subscriptionProductIds = getSubscriptionProductIds().Apple;

              //if there are no product ids, we cannot determine if user has purchased subscription
              //smth. wen wrong, so user
              if (subscriptionProductIds.length < 1) {
                //consider that user doesn't have access to pro (but don't update main variable)
                deferred.resolve(false);
                return;
              }

              // get all in app purchases for all subscription products
              var inAppPurchasesForProductId = _.filter(
                receipt.inAppPurchases,
                function(item) {
                  return _.contains(subscriptionProductIds, item.productId);
                }
              );

              //if there were no inapp purchases for product, user doesn't have access to pro
              if (inAppPurchasesForProductId.length === 0) {
                updateCommonVarHasUserAccesToPro(false, deferred);
                return;
              }

              // sort all inapp purchases by date of expiry
              var sortedProductIds = _.sortBy(
                inAppPurchasesForProductId,
                function(item) {
                  return new Date(item.subscriptionExpirationDate).getTime();
                }
              );

              // now latest purchase is first one
              sortedProductIds.reverse();

              var latestProduct = sortedProductIds[0];

              var now = new Date().getTime();
              // check if uses has active subscription
              if (
                new Date(latestProduct.subscriptionExpirationDate).getTime() >
                now
              ) {
                //this method was called because information was received that user doesn't have access to PRO when in fact he has,
                //so there was information mismatch on server-side, so we need to update server side with proper data
                recordSubscriptionOnServerForApple(productCode, receiptData);
                updateCommonVarHasUserAccesToPro(true, deferred);

                //subscription was probably renewed we have to record this as "purchase"
                trackPurchase(latestProduct.productId);
              } else {
                updateCommonVarHasUserAccesToPro(false, deferred);
                return;
              }
            })
            .catch(function(err) {
              console.log("Error getting local receipt bundle: ", err);
              updateCommonVarHasUserAccesToPro(false, deferred);
              return;
            });
        })
        .catch(function(err) {
          // there was some error in getting receipt
          console.log(err);
          updateCommonVarHasUserAccesToPro(false, deferred);
        });
    } else {
      //inAppPurchases are not defined, something went wrong
      updateCommonVarHasUserAccesToPro(false, deferred);
    }
  };

  /**
   * @summary This method decides whether Apple receipt needs to be refreshed, and if it does, it refreshes it
   * @param {string} productCode Product (code) for which we are checking subscription status
   * @param {any} deferred Promise to resolve
   */
  var refreshAppleReceiptIfNeeded = function(productCode, deferred) {
    var shouldRefreshRecipt = false;

    var subscriptionInfo = mad.local.loadVariableParse("SubscriptionInfo"); //if no data was found in local storage, empty array is returned

    if (subscriptionInfo) {
      if (typeof subscriptionInfo.WasUserRecentlySubscribed === "boolean") {
        if (subscriptionInfo.WasUserRecentlySubscribed) {
          shouldRefreshRecipt = true;
        }
      }
    }

    if (shouldRefreshRecipt) {
      refreshAppleReceiptAndValidateIt(productCode, deferred);
    } else {
      //receipt doesn't need refreshing and user doesn't have access to pro
      updateCommonVarHasUserAccesToPro(false, deferred);
    }
  };

  /**
   * @summary This method validates iOS receipts and checks in-app purchases so that we can see if subscription purchase for the product was made
   * @param {string} productCode Product (code) for which we are checking subscription status
   * @param {any} deferred
   */
  var validateLocalReceiptOniOs = function(productCode, deferred) {
    if (inAppPurchase) {
      inAppPurchase
        .getReceiptBundle()
        .then(function(receipt) {
          //if receipt doesn't exist, probably there was some err, or user hasn't made any purchases
          // so we maybe need to refresh apple receipt
          if (!receipt) {
            refreshAppleReceiptIfNeeded(productCode, deferred);
            return;
          }

          // if there are no purchases, user doesn't have access to pro
          if (receipt.inAppPurchases.length === 0) {
            refreshAppleReceiptIfNeeded(productCode, deferred);
            return;
          }

          var subscriptionProductIds = getSubscriptionProductIds().Apple;

          //if there are no product ids, we cannot determine if user has purchased subscription
          //smth. wen wrong, so user
          if (subscriptionProductIds.length < 1) {
            //consider that user doesn't have access to pro (but don't update main variable)
            deferred.resolve(false);
            return;
          }

          // get all in app purchases for all subscription products
          var inAppPurchasesForProductId = _.filter(
            receipt.inAppPurchases,
            function(item) {
              return _.contains(subscriptionProductIds, item.productId);
            }
          );

          //if there were no inapp purchases for product
          //so we maybe need to refresh apple receipt
          if (inAppPurchasesForProductId.length === 0) {
            refreshAppleReceiptIfNeeded(productCode, deferred);
            return;
          }

          // sort all inapp purchases by date of expiry
          var sortedProductIds = _.sortBy(inAppPurchasesForProductId, function(
            item
          ) {
            return new Date(item.subscriptionExpirationDate).getTime();
          });

          // now latest purchase is first one
          sortedProductIds.reverse();
          var latestProduct = sortedProductIds[0];

          var now = new Date().getTime();
          var subscriptionTime = new Date(
            latestProduct.subscriptionExpirationDate
          ).getTime();

          // check if user has active subscription
          if (subscriptionTime > now) {
            //this method was called because information was received that user doesn't have access to PRO when in fact he has,
            //so there was information mismatch on server-side, so we need to update server side with proper data
            recordSubscriptionUpdateOnServerForApple(
              productCode,
              deferred,
              latestProduct
            );
            updateCommonVarHasUserAccesToPro(true, deferred);

            //subscription was probably renewed we have to record this as "purchase"
            trackPurchase(latestProduct.productId);
          } else {
            // user has bought subscription for the product, but it has expired,so we maybe need to refresh apple receipt
            refreshAppleReceiptIfNeeded(productCode, deferred);
          }
        })
        .catch(function(err) {
          console.log("Error getting local receipt bundle: ", err);
          //there has been some error, we cannot validate receipt
          // so we maybe need to refresh apple receipt
          refreshAppleReceiptIfNeeded(productCode, deferred);
        });
    } else {
      // there has been some error, we cannot validate receipt
      // so we maybe need to refresh apple receipt
      refreshAppleReceiptIfNeeded(productCode, deferred);
    }
  };

  /**
   * @summary Checks if user has access to Pro by checking Apple's receipt which contains user's in-app purchases
   * @param {string} productCode Product (code) for which we are checking subscription status
   * @param {any} deferred Promise to resolve
   */
  var checkIfUserHasAccessToProOniOs = function(productCode, deferred) {
    //calls method to validate Apple receipt and check in-app purchases
    validateLocalReceiptOniOs(productCode, deferred);
  };

  /**
   * @summary Checks if user has access to Pro based on Platform (this is handled differently on Android & iOS)
   * @param {string} productCode Product (code) for which we are checking status of the subscription
   * @param {any} deferred Promise to resolve
   */

  var checkIfUserHasProBasedOnPlatform = function(productCode, deferred) {
    if (window.navigator.simulator) {
      updateCommonVarHasUserAccesToPro(false, deferred);
    } else {
      if (device.platform === "Android") {
        //so according to server user doesn't have pro and user is on Android
        //Our server is in sync with Google, so this means that user really doesn't have PRO
        updateCommonVarHasUserAccesToPro(false, deferred);
      } else if (device.platform === "iOS") {
        //our server doesn't always have to be in sync with Apple
        // so we have to make sure that user actually doesn't have subscription, and we'll double check this with Apple
        checkIfUserHasAccessToProOniOs(productCode, deferred);
      }
    }
  };

  /**
   * @summary  This method checks server whether user has subscription or not for the product
   * @param {string} productCode Product (code) for which we are checking status of the subscription
   * @param {any} deferred Promise to resolve
   */
  var checkIfUserHasAccessToProOnServer = function(productCode, deferred) {
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/CheckIfUserHasActiveSubscriptionForProdcut",
      akcija: "GET"
    };

    var options = {
      productCode: productCode
    };

    mad.data
      .load(route, options, false, true)
      .done(function(userHasAccessToPro) {
        if (userHasAccessToPro) {
          console.log("According to server user has access to pro");
          updateCommonVarHasUserAccesToPro(true, deferred);
        } else {
          console.log("According to server user doesn't have access to pro");
          checkIfUserHasProBasedOnPlatform(productCode, deferred);
        }
      })
      .fail(function(data) {
        console.log("There has been some err on server:", data);
        checkIfUserHasProBasedOnPlatform(productCode, deferred);
      });
  };

  /**
   * @description This method check if user has pro based on data initially received from server (this data is received every time user opens the app)
   * @param {string} productCode Product (code) for which we are checking subscription status
   * @param {any} deferred Promise top resolve
   */
  var checkUserDataIfUserHasAccessToPro = function(productCode, deferred) {
    var subscriptionInfo = mad.local.loadVariableParse("SubscriptionInfo"); //if no data was found in local storage, empty array is returned

    if (subscriptionInfo) {
      var hasAccessToPro = subscriptionInfo.UserHasActiveSubscription;
      //if variable is actually boolean
      if (typeof hasAccessToPro === "boolean") {
        if (hasAccessToPro) {
          // our server says user has pro, so that's it
          updateCommonVarHasUserAccesToPro(true, deferred);
        } else {
          checkIfUserHasProBasedOnPlatform(productCode, deferred);
        }
      } else {
        //there was some trouble getting user data
        //let's check on server
        checkIfUserHasAccessToProOnServer(productCode, deferred);
      }
    } else {
      //there was some trouble getting user data
      //let's check on server
      checkIfUserHasAccessToProOnServer(productCode, deferred);
    }
  };

  var checkIfUserHasPro = function(deferred) {
    var productCode = "madbarzpro";
    checkUserDataIfUserHasAccessToPro(productCode, deferred);
  };

  /*=================================================================
   * 
   * ****************** PURCHASING SUBSCRIPTION *******************
   * 
   *=================================================================*/

  /**
   * @summary This method handles recording purchase of subscription product on server for iOS purchase
   * @param {Object} receipt This is Apple receipt, it contains all in-app purchase of the user and all the necessary data about them
   * @param {string} prodcutCode Product (code) for which subscription (product) is purchased
   * @param {any} deferred Promise to resolve
   */
  var handlePurchaseOfSubscriptionOnServerForIos = function(
    receipt,
    productCode,
    deferred
  ) {
    recordSubscriptionOnServerForApple(productCode, receipt, deferred);
  };

  /**
   * @summary This method handles recording purchase of subscription product on server for iOS purchase
   * @param {Object} order This is object containing all necessary info about this particular order
   * @param {string} prodcutCode Product (code) for which subscription (product) is purchased
   * @param {any} deferred Promise to resolve
   */
  var handlePurchaseOfSubscriptionOnServerForAndroid = function(
    order,
    productCode,
    deferred
  ) {
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/PurchaseSubscriptionOnAndroid",
      akcija: "POST"
    };

    var options = {
      Data: order
    };
    mad.data
      .load(route, options, false, true)
      .done(function(data) {
        console.log("Data from buying subscription on server is:", data);
        if (data) {
          mad.common.hideLoader();
          updateCommonVarHasUserAccesToPro(true, deferred);
        } else {
          mad.common.hideLoader();
          updateCommonVarHasUserAccesToPro(false, deferred);
        }
      })
      .fail(function(data) {
        console.log(data);
        mad.common.hideLoader();
        updateCommonVarHasUserAccesToPro(false, deferred);
      });
  };

  var handlePurchaseOfSubscriptionOnServer = function(
    order,
    productCode,
    deferred
  ) {
    //handle purchase on iOS
    if (device.platform === "iOS") {
      handlePurchaseOfSubscriptionOnServerForIos(
        order.receipt,
        productCode,
        deferred
      );
    } else if (device.platform === "Android") {
      //handle purchase on Android
      handlePurchaseOfSubscriptionOnServerForAndroid(
        order.receipt,
        productCode,
        deferred
      );
    } else {
      //there has been some err
      deferred.resolve(false);
    }
  };

  /**
   * @summary This method handles in-app purchase of subscription
   * @param {string} id Id of subscription product
   * @param {string} prodcutCode Product (code) for which subscription (product) is purchased
   * @param {any} deferred
   */
  var purchaseSubscription = function(id, prodcutCode, deferred) {
    // in-app purchases are only available on  iOS  and Android
    if (device.platform === "iOS" || device.platform === "Android") {
      if (inAppPurchase) {
        inAppPurchase
          .subscribe(id)
          .then(function(data) {
            console.log(data);
            //in-app purchase was successful, record purchase on server
            handlePurchaseOfSubscriptionOnServer(data, prodcutCode, deferred);
          })
          .catch(function(err) {
            console.log(err);
            deferred.resolve(false);
          });
      } else {
        console.log("InApp purchases are not defined!");
        deferred.resolve(false);
      }
    } else {
      console.log("InApp purchases are not available!");
      deferred.resolve(false);
    }
  };

  /**
   * @summary This method gives user subscription product for free
   * @param {string} id Id of subscription product
   * @param {string} prodcutCode Product (code) for which subscription (product) is given
   * @param {any} deferred
   */
  var purchaseSubscriptionForFree = function(id, productCode, deferred) {
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/PurchaseSubscriptionForFree",
      akcija: "POST"
    };

    var options = {
      SubscriptionProductId: id,
      ProductCode: productCode
    };

    mad.data
      .load(route, options, false, true)
      .done(function() {
        mad.common.hideLoader();

        updateCommonVarHasUserAccesToPro(true, deferred);
      })
      .fail(function(data) {
        console.log(data);
        mad.common.hideLoader();
        deferred.resolve(false);
      });
  };

  /**
   * @summary This method gets all possible subscription products ids for some product (basically all the subscription products that user can purchase)
   * @param {string} productCode Product (code) for which we are getting all subscription products
   */
  var getSubscriptionProductIdsFromServer = function(productCode) {
    console.log("Getting subscription product ids");
    var route = {
      putanja:
        mad.common.variables.serverUrl +
        "api/Subscription/GetIdsOfAllSubscriptionProductsForProduct",
      akcija: "GET"
    };

    var options = {
      productCode: productCode
    };

    mad.data
      .load(route, options, false, true)
      .done(function(data) {
        //we have to save this data so it can be available whole time
        mad.common.variables.subscriptionProductIds.Apple = data.AppleIds;
        mad.common.variables.subscriptionProductIds.Google = data.GoogleIds;
        mad.common.variables.subscriptionProductIds.Web = data.WebIds;
        mad.local.saveVariableParse(
          "subscriptionProductIds",
          mad.common.variables.subscriptionProductIds
        );
        console.log(
          "got subscription product ids: ",
          mad.common.variables.subscriptionProductIds
        );
      })
      .fail(function(data) {
        console.log(data);
        //if request fails for some reason, just save object with empty lists
        mad.local.saveVariableParse(
          "subscriptionProductIds",
          mad.common.variables.subscriptionProductIds
        );
      });
  };

  /*=================================================================
     * 
     * ****************** RESTORING SUBSCRIPTION *******************
     * 
     *=================================================================*/

  /**
   * @summary This method handles restoring of purchased items
   * @param {string} productCode Product (code) for which restoring of purchase is possible
   * @param {any} deferred Promise to resolve
   */
  function restorePurchases(productCode, deferred) {
    if (inAppPurchase) {
      inAppPurchase
        .restorePurchases()
        .then(function(data) {
          handleRestroingOfUswPurchase(data);
          refreshAppleReceiptAndValidateIt(productCode, deferred);
        })
        .catch(function(err) {
          console.log(err);
        });
    }

    function handleRestroingOfUswPurchase(purchases) {
      for (var i = 0; i < purchases.length; i++) {
        if (
          purchases[i].productId === "com.madbarz.madbarzapp.unlimitedsaved"
        ) {
          mad.inAppPurchases.handlePurchaseOfUnlimitedSavedOnServer(
            purchases[i],
            true
          );
          break;
        }
      }
    }
  }

  return {
    checkIfUserHasPro: checkIfUserHasPro,
    purchaseSubscription: purchaseSubscription,
    purchaseSubscriptionForFree: purchaseSubscriptionForFree,
    getSubscriptionProductIdsFromServer: getSubscriptionProductIdsFromServer,
    restorePurchases: restorePurchases
  };
})();
