var mad = mad || {};

mad.viewModels = mad.viewModels || {};

mad.viewModels.planStat = (function() {
  "use strict";

  var viewModel = kendo.observable({
    // properties or functions we want to observe and expose to html
    common: mad.common.observables,

    init: initView,
    show: showView,
    afterShow: afterShow,
    beforeShow: beforeShow,
    beforeHide: beforeHide,

    leftButtonAction: mad.common.goBack,

    points: "",
    time: "",
    progress: "",
    bestExerciseValue: "", // postavljeno u madbarzWorkout
    bestExerciseName: "", // postavljeno u madbarzWorkout
    finished: "", // postavljeno u madbarzWorkout

    dataLoaded: false,

    defaultData: {
      Stats: {
        ActivatedMuscleIds: []
      }
    },

    viewData: {},

    percent: "_",
    weeks: "_",
    workoutsDone: 0,
    workoutsLeft: 0,
    avgWktDuration: 0,
    totalHrs: 0,
    timeSize: "",
    imaExercisea: false,

    binded: false
  });

  // show muscles when data arrives
  viewModel.bind("change", function(e) {
    if (e.field == "dataLoaded") {
      setTimeout(function() {
        console.log("loaded", viewModel.dataLoaded);
        if (viewModel.dataLoaded) {
          if (mad.common.$actualViewID == "#planStat") {
            // samo ako je na pravom viewu jer se moglo otic negdje drugdje dok su se loadali podaci
            mad.common.hideLoader();
            prepare();
          } else {
            console.log(
              " x x x -- loadani podaci za stat plana al se vec otislo s planStat viewa"
            );
          }
        }
      }, 1);
    }
  });

  function initView(e) {
    console.log("init planStat -------------------");
    $("#planStat-list")
      .kendoMobileListView()
      .data("kendoMobileListView"); // pretvori u pravu listu
  }

  function showView(e) {
    console.log("show planStat ----- -");
    mad.common.setActualView();
    mad.common.addViewToHistory();
    mad.common.stretchViewContentToBeElastic();

    kendo.bind($(mad.common.$actualViewID), viewModel);

    // show default data before information loaded
    // mad.common.showLoader();

    //$("#planStat-muscles").show();

    viewModel.set("workoutsDone", 0);
    viewModel.set("workoutsLeft", 0);
    viewModel.set("percent", 0);
    viewModel.set("weeks", "");
    viewModel.set("avgWktDuration", 0);
    viewModel.set("totalHrs", 0);
    viewModel.set("timeSize", "big");

    viewModel.set("viewData", viewModel.defaultData);
    setViewTitle("");
    showMuscles();
    setHeights();
    drawGraph(0);

    var contentWidth = $(window).width();
    $("#planStat .margin-wrapper").css("width", contentWidth);

    mad.common.animate();
  }

  function afterShow(e) {
    console.log("after show planStat --");
  }

  function beforeShow(e) {
    $("#planStat").removeClass("transBack");
  }

  function beforeHide(e) {
    //$("#planStat-muscles").hide();
    $("#planStat").addClass("transBack");
  }

  /* not in viewModel Context  - - - - - -  */
  function setViewTitle(naslov) {
    mad.common.actualView.title = naslov;
    mad.common.actualView.header
      .find(".km-navbar")
      .data("kendoMobileNavBar")
      .title(naslov);
  }

  function prepare() {
    mad.common.deAnimate();

    viewModel.set("viewData", mad.models.plansProgress.data()[0]);
    //console.log(mad.viewModels.planStat.viewModel.get("viewData"));

    var naslov = mad.viewModels.planStat.viewModel.get("viewData.Name");
    setViewTitle(naslov);

    viewModel.set(
      "workoutsDone",
      viewModel.get("viewData.Progress.FinishedWorkoutCount")
    );
    viewModel.set(
      "workoutsLeft",
      viewModel.get("viewData.Progress.TotalWorkoutCount") -
        viewModel.get("viewData.Progress.FinishedWorkoutCount")
    );
    viewModel.set(
      "percent",
      Math.round(
        (viewModel.get("viewData.Progress.FinishedWorkoutCount") /
          viewModel.get("viewData.Progress.TotalWorkoutCount")) *
          100
      )
    );
    viewModel.set(
      "weeks",
      viewModel.get("viewData.Progress.FinishedWeekCount") +
        " / " +
        viewModel.get("viewData.Progress.TotalWeekCount") +
        " weeks done"
    );
    viewModel.set(
      "totalHrs",
      formatTotalTime(viewModel.get("viewData.Stats.ActualDuration"))
    );
    viewModel.set(
      "avgWktDuration",
      Math.round(viewModel.get("viewData.Stats.AverageWorkoutDuration") / 60) +
        "min"
    );

    showWorkout();
    showMuscles();

    setHeights();
    drawGraph(viewModel.get("percent"));

    mad.common.animate();
  }

  function showWorkout() {
    var template = kendo.template($("#plan-stat-hist-list-template").html(), {
      useWithBlock: false
    });
    var exercises = viewModel.get("viewData.Stats.ExerciseStats");
    var result = "";
    if (exercises.length > 0) {
      result = template(exercises); //Pass the data to the compiled template
      $("#planStat-list").html(result);
      viewModel.set("imaExercisea", true);
    } else {
      viewModel.set("imaExercisea", false);
    }
  }

  function showMuscles() {
    var musclesTemplate = kendo.template(
      $("#plan-stat-muscles-template").html()
    );
    var musclesHTML = musclesTemplate(mad.viewModels.planStat.viewModel);
    $("#planStat-muscles").html(musclesHTML);

    var muscles = viewModel.get("viewData.Stats.ActivatedMuscleIds");

    console.log("aktivirani misici", muscles);

    musclesTemplate = kendo.template($("#MuscleMapTemplate").html(), {
      useWithBlock: false
    });
    musclesHTML = musclesTemplate(muscles);
    $("#planStat-muscles-imgs").html(musclesHTML);
  }

  function setHeights() {
    var contentHeight = parseInt(mad.mobileApp.view().content.height());
    var contentWidth = parseInt(mad.mobileApp.view().content.width());
    var reduce = 25 / 100; //percentage
    var muscleManContainerHeight = contentHeight - contentHeight * reduce;

    $("#planStat-muscles .muscles-container").css(
      "height",
      muscleManContainerHeight
    );
    mad.common.scaleMuscleMan2(muscleManContainerHeight, contentWidth, 10);

    mad.common.scrollViewToTop();
  }

  function drawGraph(percent) {
    var selector = "#percentGraph";
    var w = 120,
      h = 120;
    drawPercent(selector, percent, w, h);
  }

  function formatTotalTime(sec) {
    var days = Math.floor(sec / 86400);
    var hours = Math.floor((sec % 86400) / 3600);
    var minutes = Math.floor(((sec % 86400) % 3600) / 60);
    var seconds = ((sec % 86400) % 3600) % 60;
    var output = "";
    var outputLen = 0;

    if (days > 0) {
      output = days + "d " + hours + "h " + minutes + "m";
    } else if (hours > 0) {
      output = hours + "h " + minutes + "min";
    } else if (minutes > 0) {
      output = minutes + "min";
    } else {
      output = seconds + "sec";
    }

    outputLen = output.length;
    if (outputLen < 6) {
      viewModel.set("timeSize", "big");
    } else if (outputLen < 10) {
      viewModel.set("timeSize", "medium");
    } else {
      viewModel.set("timeSize", "small");
    }

    $("#planStat-top .value")
      .removeClass("small")
      .removeClass("medium")
      .removeClass("big")
      .addClass(viewModel.get("timeSize"));
    return output;
  }

  // ---------- d3 ------------

  function drawPercent(selector, percent, width, height) {
    var w = width,
      h = height,
      pathBackground,
      pathChart,
      middleCount,
      posto;

    var ratio = percent / 100;
    var outerRadius = w / 2 - 10;
    var innerRadius = w / 2 - 22;

    var arc = d3.svg
      .arc()
      .innerRadius(innerRadius)
      .outerRadius(outerRadius)
      .startAngle(0)
      .endAngle(2 * Math.PI);

    var arcLine = d3.svg
      .arc()
      .innerRadius(innerRadius)
      .outerRadius(outerRadius)
      .startAngle(0);

    var d3container = d3.select(selector);
    var svg = d3container.select("svg");

    if (svg.empty()) {
      svg = d3container
        .append("svg")
        .attr({
          width: w,
          height: h,
          class: "postoGraf"
        })
        .append("g")
        .attr({
          transform: "translate(" + w / 2 + "," + h / 2 + ")"
        });
      pathBackground = svg.append("path").attr("class", "pathBackground");
      pathChart = svg.append("path").attr("class", "pathChart");
      middleCount = svg.append("text").attr("class", "middleCount");
      posto = svg
        .append("text")
        .text("%")
        .attr("class", "posto");
    } else {
      pathBackground = svg.select(".pathBackground");
      pathChart = svg.select(".pathChart");
      middleCount = svg.select(".middleCount");
      posto = svg.select(".posto");
    }

    pathBackground.attr({ d: arc });
    pathChart.datum({ endAngle: 0 }).attr({ d: arcLine });
    middleCount
      .text(function(d) {
        return d;
      })
      .attr({
        "text-anchor": "middle",
        dy: 8,
        dx: -1
      });
    posto.attr({
      "text-anchor": "middle",
      dx: 17,
      dy: -5
    });

    var arcTween = function(transition, newAngle) {
      transition.attrTween("d", function(d) {
        var interpolate = d3.interpolate(d.endAngle, newAngle);
        var interpolateCount = d3.interpolate(0, percent);
        return function(t) {
          d.endAngle = interpolate(t);
          middleCount.text(Math.floor(interpolateCount(t)));
          return arcLine(d);
        };
      });
    };

    var animate = function() {
      pathChart
        .transition()
        .duration(1234)
        .ease("cubic")
        .call(arcTween, 2 * Math.PI * ratio);
    };
    setTimeout(animate, 333);
  }

  return {
    viewModel: viewModel
  };
})();
