(function ($, console, win) {

    var cardioStrenghtCalculateHelper = kendo.observable({
        model: window.createWorkout.model,

        updateBodyAndPercentages: function () {
            var cardio = 0;
            var strength = 0;
            var selectedExercises = cardioStrenghtCalculateHelper.model.routines.data();

            $.each(selectedExercises, function (key,exercise) {

                var numberOfTimesCardio = 0;
                var numberOfTimesPlyo = 0;
                var numberOfTimesRepetative = 0;
                var numerOfTimesStatic = 0;
                var exerciseCategories = _.pluck(exercise.Categories, 'Name');

                _.forEach(exerciseCategories, function (tempCategory) {
                    if (tempCategory.toLowerCase() == "cardio") {

                        numberOfTimesCardio++;
                    }
                    else if (tempCategory.toLowerCase() == "plyometrics") {

                        numberOfTimesPlyo++;
                    }
                    else if (tempCategory.toLowerCase() == "repetative") {

                        numberOfTimesRepetative++;

                    }
                    else if (tempCategory.toLowerCase() == "static") {
                        numerOfTimesStatic++;

                    }
                });


                if (numberOfTimesCardio > 0) {
                    cardio += 15;

                }
                else if (numberOfTimesPlyo > 0 && numberOfTimesCardio <= 0) {
                    cardio += 10;
                }

                if (numberOfTimesPlyo > 0 || numberOfTimesRepetative > 0) {
                    strength += 19;
                }
                else if (numberOfTimesPlyo <= 0 && numberOfTimesRepetative <= 0 && numerOfTimesStatic > 0) {
                    strength += 12;
                }
            });

            strengthPercentageWithoutPauses = strength;
            cardioPercentagesWithoutPauses = cardio;

            pausePercentages = cardioStrenghtCalculateHelper.updatePausePercentages(cardio, strength);

            cardio = pausePercentages.cardio;
            strength = pausePercentages.strength;

            var temp = cardioStrenghtCalculateHelper.fineTuneStrengthAndCardio(strength, cardio);

            //$scope.strength = temp.strength;
            //$scope.cardio = temp.cardio;

            if (selectedExercises.length == 0){
                temp.strength = 0;
                temp.cardio = 0;
            }

            return {
                muscle: temp.strength,
                cardio: temp.cardio,
            };

            //if ($('.muscles-visual').data("musclemap") !== null) {
            //    $('.muscles-visual').musclemap("destroy");
            //}


            //$(".muscles-visual").musclemap({
            //    muscles: muscleIds
            //});
        },//end update body and percentages
        updatePausePercentages: function (cardio, strength) {

            var shortPause = cardioStrenghtCalculateHelper.model.pauseExcersises;
            var longPause = cardioStrenghtCalculateHelper.model.pauseCycles;

            var shortPauseLimit = 70;
            var longPauseLimit = 110;

            var increaseCardio = 0;
            var decreaseCardio = 0;
            var increaseStrength = 0;
            var decreaseStrength = 0;

            var shortPauseUpperLimit = 180;
            var longPauseUpperLimit = 240;

            var percentageOfSmallPauseToAddToCardio = 2;
            var percentageOfSmallPauseToDecreaseToCardio = 1;
            var percentageOfSmallPauseToDecreaseStrength = 1;
            var percentageOfSmallPauseToIncreaseStrength = 1;

            var percentageOfLongPauseToAddToCardio = 2;
            var percentageOfLongPauseToDecreaseToCardio = 0.5;
            var percentageOfLongPauseToDecreaseStrength = 1;
            var percentageOfLongPauseToIncreaseStrength = 1;

            var cardioPercentageOfFluctuaction = 0.5;
            var strengthPercentageOfFluctuaction = 0.5;


            if (shortPause > 180) {
                shortPause = 180;
            }

            if (longPause > 360) {
                longPause = 360;
            }


            if (shortPause <= shortPauseLimit) {
                increaseCardio = (shortPauseLimit / 5 - Math.floor(shortPause / 5)) * percentageOfSmallPauseToAddToCardio;
                decreaseStrength = (shortPauseLimit / 5 - Math.floor(shortPause / 5)) * percentageOfSmallPauseToDecreaseStrength;
            } else {
                decreaseCardio = Math.floor((shortPause - shortPauseLimit) / 5) * percentageOfSmallPauseToDecreaseToCardio;
                increaseStrength = Math.floor((shortPause - shortPauseLimit) / 10) * percentageOfSmallPauseToIncreaseStrength;
            }

            if (longPause <= longPauseLimit) {
                increaseCardio += ((longPauseLimit / 10 - Math.floor(longPause / 10)) * percentageOfLongPauseToAddToCardio);
                decreaseStrength += ((longPauseLimit / 10 - Math.floor(longPause / 10)) * percentageOfLongPauseToDecreaseStrength);
            } else {
                decreaseCardio += (Math.floor((longPause - longPauseLimit) / 10) * percentageOfLongPauseToDecreaseToCardio);
                increaseStrength += (Math.floor((longPause - longPauseLimit) / 10) * percentageOfLongPauseToIncreaseStrength);
            }

            var cardioMaxFluctuaction = cardio * cardioPercentageOfFluctuaction;
            var strengthMaxFluctuaction = strength * strengthPercentageOfFluctuaction;

            if (cardio + increaseCardio - decreaseCardio < cardio - cardioMaxFluctuaction) {
                cardio -= cardioMaxFluctuaction;
            } else if (cardio + increaseCardio - decreaseCardio > 97) {
                cardio = 100;
            } else {
                cardio = cardio + increaseCardio - decreaseCardio;
            }

            if (strength + increaseStrength - decreaseStrength < strength - strengthMaxFluctuaction) {
                strength -= strengthMaxFluctuaction;
            } else if (strength + increaseStrength - decreaseStrength > 97) {
                strength = 100;
            } else {
                strength = strength + increaseStrength - decreaseStrength;
            }

            var cardioAndStrengthHelperObject = { cardio: cardio, strength: strength };

            return cardioAndStrengthHelperObject;
        }, // end updatePausePercentages

        fineTuneStrengthAndCardio:function (strength, cardio) {
            if (cardio > 60 && strength > 40) {
                var k = (cardio - 60) / 100;
                strength = strength - k * strength;
            }

            // this is just for "security" as percentages alway shoudl be [0,100]

            if (cardio > 98) {
                cardio = 100;
            }
            else if (cardio <= 0) {
                cardio = 3;
            }


            if (strength > 98) {
                strength = 100;
            }
            else if (strength <= 0) {
                strength = 3;
            }

            var cardioAndStrengthHelperObject = { cardio: cardio, strength: strength };

            return cardioAndStrengthHelperObject;
        }// end fineTuneStrengthAndCardio

    });

    $.extend(window.createWorkout, {
        cardioStrenghtCalculateHelper: cardioStrenghtCalculateHelper,
    });

})
(jQuery, console, window);