var mad = mad || {};

mad.viewModels = mad.viewModels || {};

mad.viewModels.createWorkoutExercises = (function () {

    "use strict";

    var viewModel = kendo.observable({
        // properties or functions we want to observe and expose to html
        common: mad.common.observables,
        model: window.createWorkout.model,

        init: initView,
        show: showView,
        afterShow: afterShow,

        leftButtonAction: leftButtonAction,
        rightButtonAction: handleFilter,
        footerBtnPressed: doneButton,
        resetView: resetView,
        doneBtnTxt: doneBtnTxt,
        removeTagFilter: removeTagFilter,
        searchFieldChanged: searchFieldChanged,
        doTagFiltering: doTagFiltering,
        lastFilterName: "",

        isButtonVisibleL: false,
        isButtonVisibleR: true,

        enableIfOfflineL: true,
        enableIfOfflineR: true,
        
        //tutorialNotShowed: checkTutorial,
        tutorialNotShowed: function(){return false},

        totalNumberOfExercises: 0,
        nonFilterData: [],
        tagFilterOn: false,
        firstTimeShow: true,
        counter: 0,
        timeLastAdded: 0,
        lastTimeSearched: null  

    });

    function initView(e){
        //console.log("init exercises -------------------");

        var scroller = $("#createWorkoutExercises-scroller").kendoMobileScroller().data("kendoMobileScroller");

        $("#createWorkoutExercisesScrollView").kendoMobileListView({
            dataSource: mad.models.exercisesDatasource,
            autoBind: false,
            template: function(data) {
                if (mad.common.isNullOrEmpty(data)){
                    console.log("nema nesto podataka");
                }else{
                    //console.log("podaci za exer", data);
                    var tpl = kendo.template($("#CreateWorkoutAddExercisesTemplate").html(), {useWithBlock:false});
                    return tpl(data);
                }
            },

            endlessScroll: false,
            loadMore:false
        }).kendoTouch({
            filter: "li, .add, .toInstructions",
            tap: exerciseTapped
        });

        // for lazy loading images
        bLazy = new Blazy({
            container: '#createWorkoutExercisesScrollView', // Default is window
            offset: 22, // Loads images x px before they're visible (default 100px)
            success: function(ele){
                // Image has loaded
                console.log("loaded", $(ele));
            }
            , error: function(ele, msg){
                console.log("error loading img", ele, msg);
                if(msg === 'missing'){
                    // Data-src is missing
                }
                else if(msg === 'invalid'){
                    // Data-src is invalid
                }
            }
        });

        scroller.bind("scroll", function(e) {

            console.log("bLazy.revalidat");
            bLazy.revalidate();

        });


    }

    function showView(e){
        console.log("show add exercises ----- -");

        if (this.tutorialNotShowed()){
            console.log("don't go further skip rest of show view");
            return;
        }

        mad.common.setActualView();
        //mad.common.addViewToHistory();
        setupView();

        // privremena visina dok se sve ne ucita
        $("#createWorkoutExercises-scroller").css("height", 464);

        kendo.bind($(mad.common.$actualViewID), viewModel);

        if (viewModel.get("firstTimeShow")){
            viewModel.set("firstTimeShow", false);
            var view = $("#createWorkoutExercisesScrollView").data("kendoMobileListView");
            //console.log("prije listview refresh");

            if (view) view.refresh();
        }

        // show last filtername
        var filterName = viewModel.get("lastFilterName");
        if (filterName == "All" || filterName == "") {
            $('#workoutsCreateAddExercisesFilterTags span').text("filter");
        } else {
            $('#workoutsCreateAddExercisesFilterTags span').text("filter: " + filterName);
        }

        bLazy.revalidate(); // to show img before scrolling

    }

    function afterShow() {
        console.log("after show add exercises --");

        $("#createWorkoutMain .km-footer .dButton").removeClass("preloader");

        if (this.tutorialNotShowed()){
            console.log("after show - don't go further - show tutorial");
            setTimeout(function () {
                mad.mobileApp.navigate("ui/tutorials/create-tutorial.html");
            }, 22);
            return true;
        }

        setTimeout(function() {
            // da se sakrije kbd i ucitaju podaci jer ce krivo izracunat visinu
            var contentHeight = parseInt(mad.mobileApp.view().content.height());
            $("#createWorkoutExercises-scroller").css("height", contentHeight);
        }, 444);

        mad.googleAnalytics.trackView("Create Workout - Add Exercises");

    }

    function checkTutorial(){
        return mad.common.checkTutorial("CreateTutorialShown");
    }

    function leftButtonAction(){
        //mad.mobileApp.navigate("ui/workouts/createWorkout/Name.html", "slide:left reverse");
    };

    function doneButton(){
        if(!mad.common.isNullOrEmpty($('#createWorkoutMain-list').data("kendoMobileListView"))) {
            $("#createWorkoutMain-list").data("kendoMobileListView").refresh();
        }

        var reminder = localStorage.getItem("tutorialCreateWorkout");
        if (reminder === null || reminder === "undefined") {
            mad.mobileApp.navigate("ui/workouts/createWorkout/Tips.html");
        } else {
            //routing.navigateBack();
            mad.mobileApp.navigate("ui/workouts/createWorkout/Main.html");
        }
    }

    function searchFieldChanged(e) {
        e.preventDefault();

        setTimeout(function() {
            doNameFiltering();
        }, 444);

    }

    function resetView(){

        // handlanje dohvata podataka iz cachea ili sa servera ako nisu cacheirani
        var offlineData = mad.models.exercisesDatasource.offlineData();

        console.log("resetView, broj offline spremljenih podataka", offlineData.length);

        if (mad.common.isNullOrEmpty(offlineData) || offlineData.length < 79){
            mad.models.exercisesDatasource.online(true);
            console.log("(madbarz exercises) podaci se dohvacaju sa servera");
        }else{
            mad.models.exercisesDatasource.online(false);
            console.log("(madbarz exercises) podaci se dohvacaju iz keša");
        }

        //console.log("online state for exercises is", mad.models.exercisesDatasource.online());

        mad.models.exercisesDatasource.read().then(function(){
            $('#workoutsCreateAddExercisesFilter').val('');
            removeTagFilter();
        });

    }

    function handleFilter(e) {
            setTimeout(function () {
                $('#workoutsCreateAddExercisesTagFilterActionsheet').data("kendoMobileActionSheet").open($('#workoutsCreateAddExercisesFilterTags'));
            }, 0);
    }

    function doneBtnTxt(){
    }

    function doTagFiltering(filterName, muscleIdsFilter) {

        var data = this.get("nonFilterData");
        if (data.length === 0) return null;

        $("#createWorkoutExercises-scroller").data("kendoMobileScroller").reset();
        console.log("filter name je", filterName);
        viewModel.set("lastFilterName", filterName);

        var nameFilter = $('#workoutsCreateAddExercisesFilter').val();

        if (nameFilter && filterName != "All") {
            resetNameFilter();
        }

        mad.googleAnalytics.trackEvent("Used Filter", "Click - Exercise Filter in CW", filterName);

        if (filterName == "All") {
            $('#workoutsCreateAddExercisesFilterTags span').text("filter");

            //removeTagFilter(false);
            removeTagFilter();

            bLazy.revalidate(); // to show img before scrolling
            return;
        } else {
            $('#workoutsCreateAddExercisesFilterTags span').text("filter: " + filterName);
        }


        var filteredData = [];
        for (var i = 0; i < data.length; i++) {
            var muscleIdsOfExercise = data[i].MuscleIds;
            for (var j = 0 ; j < muscleIdsOfExercise.length; j++) {
                var exists = false;
                for (var k = 0; k < muscleIdsFilter.length; k++) {
                    if (muscleIdsOfExercise[j] == muscleIdsFilter[k]) {
                        exists = true;
                        break;
                    }
                }
                if (exists) {
                    filteredData.push(data[i]);
                    break;
                }
            }
        }
        console.log("filtrirani podaci", filteredData);

        mad.models.exercisesDatasource.data(filteredData);
        this.set('tagFilterOn', true);

        bLazy.revalidate(); // to show img before scrolling
    }

    function removeTagFilter(param){
        console.log("removeTagFilter", param);
        mad.viewModels.createWorkoutExercises.viewModel.set('tagFilterOn', false);
        $('#workoutsCreateAddExercisesFilterTags').addClass('filterNotActive');

        viewModel.set("lastFilterName", "All");
        $('#workoutsCreateAddExercisesFilterTags span').text("filter");

        mad.models.exercisesDatasource.data(mad.viewModels.createWorkoutExercises.viewModel.get("nonFilterData"));
        if (typeof param == "undefined") {
            resetNameFilter();
        }
    }

    function resetNameFilter(){
        //console.log("resetNameFilter");

        var filter = $('#workoutsCreateAddExercisesFilter').val('');
        var filterRez = mad.models.exercisesDatasource.filter({});
        //console.log("filter je vratio", filterRez);

        var view = $("#createWorkoutExercisesScrollView").data("kendoMobileListView");
        if (view) view.refresh();

        if(bLazy){
            bLazy.revalidate(); // to show img before scrolling
        }

        var scroller = $("#createWorkoutExercises-scroller").data("kendoMobileScroller");
        if (scroller) scroller.reset();
    }

    //** ---- not in viewModel Context - - - - - - - - - - -  - - - - - -
    function setupView(){
        mad.common.stretchView();
        setViewTitle();
        mad.common.setResolutionForView(parseInt(mad.mobileApp.view().content.width()), parseInt(mad.mobileApp.view().content.height()));
        mad.common.setLangForView();

    };

    function setViewTitle(){
        var naslov = mad.common.translate("ui", "header.title");
        mad.common.actualView.title = naslov;
        mad.common.actualView.header
            .find(".km-navbar")
            .data("kendoMobileNavBar").title(naslov);
    };

    function doNameFiltering () {
        var addExercises = mad.viewModels.createWorkoutExercises.viewModel;

        var filter = $('#workoutsCreateAddExercisesFilter').val();
        var fl = filter.length;
        if (fl < 3){
            if (fl == 0){
                resetNameFilter();
            }
            return;
        }

        if (addExercises.get("tagFilterOn")) {
            removeTagFilter(false);
        }

        mad.models.exercisesDatasource.filter({
            field: "Name",
            operator: function (item, value) {
                var wordsInExerciseName = item.toLowerCase().split(" ");
                return chechkIfWordsStartWith(wordsInExerciseName, value.toLowerCase());

            },
            value: filter });
        $("#createWorkoutExercises-scroller").data("kendoMobileScroller").reset();
        $("#createWorkoutExercisesScrollView").data("kendoMobileListView").refresh();
        bLazy.revalidate(); // to show img before scrolling

    }

    function exerciseTapped(e) {

        e.preventDefault(); // da ne probija

        var element = e.touch.target;
        var parentElement = $(e.touch.target.parent());
        var uidElement = element.closest("li");
        var exercise = mad.models.exercisesDatasource.getByUid(uidElement.attr('data-uid'));
        //potrebno za video
        mad.viewModels.workoutVideo.viewModel.set("selectedExercise",exercise);

        if ($(element).hasClass("toInstructions")) {
            /*
            if (parentElement.hasClass("opened")){              //close item on tap (if it's opened)
                parentElement.animate({ left: 0 }, "fast");
                parentElement.removeClass("opened");
            }else{
                mad.viewModels.workoutVideo.viewModel.openVideoModal();
            }
            */
            mad.viewModels.workoutVideo.viewModel.openVideoModal();

        } else if ($(element).hasClass("add")) {
            if (mad.viewModels.createWorkoutExercises.viewModel.timeLastAdded + 300 > (new Date()).getTime()) {
                return null;
            }

            //$(e.touch.currentTarget).css('opacity', 0.5);
            $(element).css('opacity', 0.5); // drazen - zbog nove verzije kenda

            var kvacica = $(e.touch.currentTarget).next().find('.done');
            if (kvacica.css('display') != 'block') {
                kvacica.css('display', 'block');
                setTimeout(function () {kvacica.css('display', 'none');}, 2000);    // i makni je za 2 sec
            }


            mad.viewModels.createWorkoutExercises.viewModel.timeLastAdded = (new Date()).getTime();
            var id = mad.viewModels.createWorkoutExercises.viewModel.model.addExercise(exercise);
            $.ajax({
                data: {
                    id: exercise.ExerciseId
                },
                type: "GET",
                beforeSend: function (request) {
                    request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
                },
                url: mad.common.variables.serverUrl + "api/CreateRoutine/GetNumberOfRepsForExercise",
                success: function (data) {
                    var exercise2 = mad.viewModels.createWorkoutExercises.viewModel.model.routines.get(id);
                    exercise2.loading = false;
                    exercise2.Reps = data;
                    /* ide na done
                    if(!mad.common.isNullOrEmpty($('#createWorkoutMain-list').data("kendoMobileListView"))) {
                        $("#createWorkoutMain-list").data("kendoMobileListView").refresh();
                    }
                    */
                },
                error: function (data) {
                    var exercise2 = mad.viewModels.createWorkoutExercises.viewModel.model.routines.get(id);
                    exercise2.loading = false;
                    exercise2.Reps = 0;
                    /* ide na done
                    if(!mad.common.isNullOrEmpty($('#createWorkoutMain-list').data("kendoMobileListView"))) {
                        $("#createWorkoutMain-list").data("kendoMobileListView").refresh();
                    }
                    */
                }
            });
        }
    }

    function chechkIfWordsStartWith(words, value) {
        if (words) {
            for (var i = 0; i < words.length; i++) {
                if (words[i].lastIndexOf(value, 0) === 0) {
                    return true;
                }
            }
        }
        
        return false;
    }

    return {
        viewModel: viewModel
    };

}());
