(function($, console, win) {
  var model = kendo.observable({
    workoutName: "",
    pauseExcersises: 30,
    pauseCycles: 90,
    cycles: 4,
    cardioPercent: "3%",
    musclePercent: "3%",
    exerciseCounter: 0,
    isSaved: false,
    editMode: false,
    savedId: 0,
    routineId: 0,
    routines: new kendo.data.DataSource({
      data: [],
      pageSize: 50
    }),
    muscleIds: new kendo.data.DataSource({
      data: []
    }),

    addExercise: function(exercise) {
      exercise = $.extend({}, exercise);
      exercise.id = new Date().getTime() + "" + model.exerciseCounter;
      exercise.loading = true;
      exercise.OrdinalNumber = model.exerciseCounter;

      if (mad.common.isNullOrEmpty(exercise.SavedExerciseId)) {
        // za novo dodane exercise
        exercise.SavedExerciseId = null;
      }
      console.log("addExercise - exer koji se dodaje u routines", exercise);

      model.routines.add(exercise);
      model.exerciseCounter++;
      if (
        !mad.common.isNullOrEmpty(
          $("#createWorkoutMain-list").data("kendoMobileListView")
        )
      ) {
        $("#createWorkoutMain-list")
          .data("kendoMobileListView")
          .refresh();
      }
      model.createMusclesArray();
      model.updateStrenghAndCardio();
      return exercise.id;
    },

    removeExercise: function(exercise) {
      var IDtoRemove = exercise.ExerciseId;
      var nadjeno = _.find(
        mad.viewModels.createWorkoutExercises.viewModel.listOfExercises.data(),
        "ExerciseId",
        IDtoRemove
      );

      if (nadjeno) {
        nadjeno.selectedXtimes -= 1;
        if (nadjeno.selectedXtimes > 1) {
          nadjeno.displayTimes = nadjeno.selectedXtimes;
        } else {
          nadjeno.displayTimes = "";
        }
        var view = $("#createWorkoutExercisesScrollView").data(
          "kendoMobileListView"
        );
        if (view) view.refresh();
      }

      model.routines.remove(exercise);
      model.createMusclesArray();
      model.updateStrenghAndCardio();
    },

    updateStrenghAndCardio: function() {
      var helper = createWorkout.cardioStrenghtCalculateHelper.updateBodyAndPercentages();
      //console.log("helper", helper);

      model.set("cardioPercent", helper.cardio + "%");
      model.set("musclePercent", helper.muscle + "%");
    },

    createMusclesArray: function() {
      var routines = model.routines.data();
      var newMuscleIdsArray = [];
      var muscleIds = model.muscleIds.data();
      for (var i = 0; routines.length > i; i++) {
        for (var j = 0; routines[i].MuscleIds.length > j; j++) {
          var muscleId = routines[i].MuscleIds[j];
          var muscleExists = false;
          for (k = 0; newMuscleIdsArray.length > k; k++) {
            if (newMuscleIdsArray[k] == muscleId) {
              muscleExists = true;
              break;
            }
          }
          if (!muscleExists) {
            newMuscleIdsArray.push(muscleId);
          }
        }
      }
      model.muscleIds.data(newMuscleIdsArray);
    },

    clearCreateWorkout: function() {
      //clears all views and clears model
      model.set("workoutName", "");
      model.set("pauseExcersises", 30);
      model.set("pauseCycles", 90);
      model.set("cycles", 4);
      model.set("cardioPercent", "3%");
      model.set("musclePercent", "3%");
      model.set("exerciseCounter", 0);
      model.set("savedId", 0);
      model.routines.data([]);
      model.muscleIds.data([]);

      var podaci = mad.viewModels.createWorkoutExercises.viewModel.get(
        "nonFilterData"
      );
      $.each(podaci, function(idx, elem) {
        elem.selectedXtimes = 0;
        elem.displayTimes = "";
      });
      mad.viewModels.createWorkoutExercises.viewModel.listOfExercises.data(
        podaci
      );

      mad.viewModels.createWorkoutName.viewModel.set("exercisesShowed", false);
      mad.viewModels.createWorkoutExercises.viewModel.resetView();
    },

    saveWorkout: function(deferred) {
      var isUserProPromise = $.Deferred();
      var hasUserPRo = false;

      isUserProPromise.then(
        function(hasAccessToPro) {
          if (typeof hasAccessToPro === "boolean") {
            hasUserPRo = hasAccessToPro;
          }

          data = model.createWorkoutModelForWeb();
          data.HasUserAccessToPro = hasUserPRo;

          saveCustomWorkoutOnServer(data, deferred);
        },
        function(fail) {
          console.log("hasUserAccessToPro failed !!");
          data = model.createWorkoutModelForWeb();
          data.HasUserAccessToPro = hasUserPRo;

          saveCustomWorkoutOnServer(data, deferred);
        }
      );

      //check if user has access to pro or not
      mad.common.hasUserAccessToPro(isUserProPromise);
    },

    updateWorkout: function(deferred) {
      var isUserProPromise = $.Deferred();
      var hasUserPRo = false;

      isUserProPromise.then(
        function(hasAccessToPro) {
          if (typeof hasAccessToPro === "boolean") {
            hasUserPRo = hasAccessToPro;
          }
          data = model.createWorkoutModelForWeb();
          data.HasUserAccessToPro = hasUserPRo;
          data.SavedWorkoutId = model.routineId;
          updateCustomWorkoutOnServer(data, deferred);
        },
        function(fail) {
          console.log("hasUserAccessToPro failed !!");
          data = model.createWorkoutModelForWeb();
          data.HasUserAccessToPro = hasUserPRo;
          updateCustomWorkoutOnServer(data, deferred);
        }
      );

      //check if user has access to pro or not
      mad.common.hasUserAccessToPro(isUserProPromise);
    },

    removeFromSaved: function(deferred) {
      var savedId = model.get("savedId");
      $.ajax({
        url: mad.common.variables.serverUrl + "api/Saved/RemoveFromSaved",
        type: "POST",
        beforeSend: function(request) {
          request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
        },
        data: {
          Id: savedId
        }
      })
        .success(function(event) {
          model.set("isSaved", false);
          model.set("savedId", 0);
          mad.local.removeVariable("savedWorkouts");
          deferred.resolve(true);
        })
        .fail(function(event) {
          mad.alerts.error("Error removing workout from saved.");
          deferred.reject(false);
        });
    },

    getModelForStartWithoutSaving: function(deferred) {
      var data = model.createWorkoutModelForWeb();
      //console.log("getModelForStartWithoutSaving sends ", data);
      $.ajax({
        url:
          mad.common.variables.serverUrl +
          "api/CreateRoutine/CreateCustomRoutineWithoutSaving",
        type: "POST",
        beforeSend: function(request) {
          request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
        },
        data: data
      })
        .success(function(data) {
          //console.log("getModelForStartWithoutSaving got ", data);
          deferred.resolve(data);
        })
        .fail(function(event) {
          deferred.reject(false);
        });
    },

    getModelForStartAlreadySaved: function(deferred) {
      //console.log("getModelForStartAlreadySaved sends ", model.get("savedId"));
      $.ajax({
        url:
          mad.common.variables.serverUrl +
          "api/CreateRoutine/StartWorkoutFromSaved",
        type: "POST",
        beforeSend: function(request) {
          request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
        },
        data: {
          Id: model.get("savedId")
        }
      })
        .success(function(data) {
          console.log("getModelForStartAlreadySaved gets", data);

          deferred.resolve(data);
        })
        .fail(function(event) {
          deferred.reject(false);
        });
    },

    createWorkoutModelForWeb: function() {
      var routines = model.routines.data();
      var formattedRoutines = [];
      for (var i = 0; i < routines.length; i++) {
        routine = {
          ExerciseId: routines[i].ExerciseId,
          SavedExerciseId: routines[i].SavedExerciseId,
          NumberOfReps: routines[i].Reps,
          OrdinalNumberOfExercise: i + 1
        };
        formattedRoutines.push(routine);
      }
      var data = {
        Name: model.workoutName,
        ShortPause: model.pauseExcersises,
        LongPause: model.pauseCycles,
        Cardio: model.cardioPercent.replace("%", ""),
        Strength: model.musclePercent.replace("%", ""),
        NumberOfCycles: model.cycles,
        Exercises: formattedRoutines,
        HasUserAccessToPro: false
      };
      return data;
    },

    needsSaving: function() {
      var routines = model.get("routines").data();
      var needsSaving = false;
      for (var i = 0; i < routines.length; i++) {
        if (routines[i].Reps > 0) {
          needsSaving = true;
          break;
        }
      }
      if (needsSaving && !model.get("isSaved")) {
        return true;
      } else {
        return false;
      }
    },

    addSavedForEdit: function(data) {
      model.clearCreateWorkout();

      model.workoutName = data.Name;
      model.routineId = data.Id;
      model.cycles = data.NumberOfCycles;
      model.pauseExcersises = data.ShortPauseLengthInSeconds;
      model.pauseCycles = data.LongPauseLengthInSeconds;
      model.routines.data([]); // empty datasource
      _.forEach(data.Exercises, function(exercise, i) {
        exercise.OrdinalNumber = 0;
        exercise.Reps = exercise.NewValue;
        var id = model.addExercise(exercise);

        console.log("priprema execisea", exercise);
      });

      console.log("podaci za listu", model.routines.data());

      mad.mobileApp.navigate(
        "ui/workouts/createWorkout/Name.html?origin=madbarzWorkout"
      );
    },

    startWorkout: function(serverModel) {
      mad.viewModels.madbarzWorkout.viewModel.set("workoutType", "Saved");
      mad.viewModels.madbarzSavedWorkoutS.viewModel.set(
        "selectedWorkout",
        serverModel.Routine
      );
      mad.viewModels.madbarzSavedWorkoutS.viewModel.selectedWorkout.set(
        "ExercisesAndReps",
        serverModel.ExercisesAndReps
      );

      var sw = mad.viewModels.madbarzSavedWorkoutS.viewModel.get(
        "selectedWorkout"
      );
      _.assign(sw, {
        MuscleIds: model.muscleIds.data(),
        Cardio: serverModel.Routine.CardioPercentage,
        Strength: serverModel.Routine.StrengthPercentage,
        RoutineId: serverModel.Routine.Id,
        UserNameWhoCreatedTrening: mad.common.variables.username, // drazen - ovo je nedostajalo (mada se spremi u saved workouts
        Id: serverModel.SavedId // drazen - ovo je isto falilo
      });
      //console.log("saved workout before start is ",mad.viewModels.madbarzSavedWorkoutS.viewModel.get("selectedWorkout"));

      mad.mobileApp.navigate("ui/workouts/madbarz/workout/madbarzWorkout.html");
    }
  });

  var createWorkout = {
    model: model
  };

  var saveCustomWorkoutOnServer = function(data, deferred) {
    console.log("data for save is", data);
    $.ajax({
      type: "POST",
      beforeSend: function(request) {
        request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
      },

      // drazen prenesi model.routineId

      data: data,
      url:
        mad.common.variables.serverUrl +
        "api/CreateRoutine/SaveCustomRoutineNew",
      success: function(data) {
        mad.common.hideLoader();
        if (data.m_Item1 === "00000000-0000-0000-0000-000000000000") {
          if (data.m_Item2 !== "max") {
            mad.alerts.error(data.m_Item2);
          } else {
            console.log("prije otvaranja purchase modala");
            mad.viewModels.popups.viewModel.openModalWindow(
              "buySavedPRO",
              "You reached your limit of 5",
              "Premium account lets you save unlimited workouts and much more",
              "Get unlimited space!"
            );
          }
          console.log("prije deferred reject");
          deferred.reject(false);
        } else {
          model.set("isSaved", true);
          model.set("savedId", data.m_Item1);
          //console.log("save id je", data);

          mad.local.removeVariable("savedWorkouts");

          mad.facebookAnalytics.trackEvent(
            "Workout Saved",
            {
              Origin: "Create Workout"
            },
            0
          );

          deferred.resolve();
        }
      }
    }).fail(function() {
      mad.common.hideLoader();

      mad.alerts.error("Error saving workout.");
      deferred.reject();
    });
  };

  var updateCustomWorkoutOnServer = function(data, deferred) {
    console.log("data for update are", data);
    $.ajax({
      type: "POST",
      beforeSend: function(request) {
        request.setRequestHeader("Authorization", "Bearer " + mad.loginToken);
      },
      data: data,
      url:
        mad.common.variables.serverUrl +
        "api/saved/EditSavedWorkout?api-version=2.0",
      success: function(data) {
        mad.common.hideLoader();
        mad.local.removeVariable("savedWorkouts");
        deferred.resolve();
      }
    }).fail(function(e) {
      mad.common.hideLoader();
      console.log("update error", e);
      mad.alerts.error("Update Error !");
      deferred.reject();
    });
  };

  $.extend(window, {
    createWorkout: createWorkout
  });
})(jQuery, console, window);
